/********************************************************************************************************
 * @file     factory_reset.c
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2016, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/

#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"

//////////////////Factory Reset///////////////////////////////////////////////////////////////////////

static int adr_reset_cnt_idx = 0;
static int reset_cnt = 0;

extern flash_adr_layout_def flash_adr_layout;

#define SERIALS_CNT                     (5)   // must less than 7

const u8 factory_reset_serials[SERIALS_CNT * 2] = { 0, 3,    // [0]:must 0
                                                    0, 3,    // [2]:must 0
                                                    0, 3,    // [4]:must 0
                                                    3, 30,
                                                    3, 30};

#define RESET_CNT_RECOUNT_FLAG          0
#define RESET_FLAG                      0x80

void	reset_cnt_clean ()
{
	if (adr_reset_cnt_idx < 3840)
	{
		return;
	}
	flash_erase_sector ((unsigned long)flash_adr_layout.flash_adr_factory_reset_cnt);
	adr_reset_cnt_idx = 0;
}

void write_reset_cnt (u8 cnt)
{
	flash_write_page ((unsigned long)flash_adr_layout.flash_adr_factory_reset_cnt + adr_reset_cnt_idx, 1, (u8 *)(&cnt));
}

void clear_reset_cnt ()
{
    write_reset_cnt(RESET_CNT_RECOUNT_FLAG);
}

int reset_cnt_get_idx ()		//return 0 if unconfigured
{
	u8 *pf = (u8 *)flash_adr_layout.flash_adr_factory_reset_cnt;
	for (adr_reset_cnt_idx=0; adr_reset_cnt_idx<4096; adr_reset_cnt_idx++)
	{
	    u8 restcnt_bit = pf[adr_reset_cnt_idx];
		if (restcnt_bit != RESET_CNT_RECOUNT_FLAG)	//end
		{
        	if(((u8)(~(BIT(0)|BIT(1)|BIT(2)|BIT(3))) == restcnt_bit)  // the fourth not valid
        	 ||((u8)(~(BIT(0)|BIT(1)|BIT(2)|BIT(3)|BIT(4)|BIT(5))) == restcnt_bit)){  // the fifth not valid
                clear_reset_cnt();
            }else{
			    break;
			}
		}
	}

    reset_cnt_clean();

	return 1;
}

u8 get_reset_cnt_bit ()
{
	if (adr_reset_cnt_idx < 0)
	{
	    reset_cnt_clean();
		return 0;
	}
	return *((u8 *)(flash_adr_layout.flash_adr_factory_reset_cnt + adr_reset_cnt_idx));
}

void increase_reset_cnt ()
{
	u8 restcnt_bit = get_reset_cnt_bit();
	foreach(i,8){
        if(restcnt_bit & BIT(i)){
            if(i < 3){
                reset_cnt = i;
            }else if(i < 5){
                reset_cnt = 3;
            }else if(i < 7){
                reset_cnt = 4;
            }

            restcnt_bit &= ~(BIT(i));
            write_reset_cnt(restcnt_bit);
            break;
        }
	}
}

int factory_reset(void){
	u8 r = irq_disable ();
    clear_reset_cnt();

    flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_hash_id);

    flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_device_id);
    flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_srp_key);
    flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_id_info);
    flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_id_info + 0x1000);
    flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_global_state);
    flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_char_value);
//R10 clear broadcast event param.
    extern void blt_clean_broadcast_param(void);
	blt_clean_broadcast_param();
    irq_restore(r);
	return 0;
}

void factory_default ()
{
	factory_reset ();
	
	//irq_disable ();
	//cpu_sleep_wakeup (1, PM_WAKEUP_TIMER, clock_time () + 400*CLOCK_SYS_CLOCK_1MS);
}

int factory_reset_handle ()
{
    reset_cnt_get_idx();
    u8 restcnt_bit;
    restcnt_bit = get_reset_cnt_bit();
	if(restcnt_bit == RESET_FLAG){
        irq_disable();
        factory_reset();

        irq_disable ();
    #if 1
        //-REG_ADDR8(0x602) = 0x84;             //reboot
        REG_ADDR8(0x6F) = 0x20;                // rst all(act as power on reset)
    #else
        cpu_sleep_wakeup (1, PM_WAKEUP_TIMER, clock_time() + 5*CLOCK_SYS_CLOCK_1MS);
    #endif
        while (1);
	}else{
        increase_reset_cnt();
	}
	return 0;
}

int factory_reset_cnt_check ()
{
    static u8 clear_st = 3;
    static u32 reset_check_time;

    if(0 == clear_st) return 0;

    if(3 == clear_st){
        clear_st--;
        reset_check_time = factory_reset_serials[reset_cnt*2];
    }

	if((2 == clear_st) && clock_time_exceed(0, reset_check_time*1000*1000)){
	    clear_st--;
	    reset_check_time = factory_reset_serials[reset_cnt*2 + 1];
	    if(3 == reset_cnt || 4 == reset_cnt){
            increase_reset_cnt();
        }
	}

	if((1 == clear_st) && clock_time_exceed(0, reset_check_time*1000*1000)){
	    clear_st = 0;
        clear_reset_cnt();
	}

	return 0;
}

void kick_out(){
    //factory_reset();
}
