/********************************************************************************************************
 * @file     version.h 
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2010, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/

#pragma once

/* Enable C linkage for C++ Compilers: */
#if defined(__cplusplus)
extern "C" {
#endif


// can't use "enum" here, because cstartup.S not support

//#define   BUILD_VERSION  	"Revision: 45:49M"
//#define   BUILD_TIME  		"2012-07-24-21:37:43"

//firmware_revision[6]   = "3.16.0";
#define Major             3
#define Minor             16
#define Revision          0

//NOTE: The compile will report error when assert the validity of Macro(Major, Minor, Revision) in this file,
//because this file is included in *.S file for assembler, so we move the assert to app.c instand.
//STATIC_ASSERT(Major <= 9);
//STATIC_ASSERT(Minor <= 99);
//STATIC_ASSERT(Revision <= 9);

#define MAJOR(major)                             (major << 4)
#define MINOR(minor)                             ((minor / 10) | ((minor % 10) << 12))
#define REVISION(revision)                       (revision << 8)

#define VERSION_GET(major, minor, revision)      (MAJOR(major) | MINOR(minor) | REVISION(revision))

#define FW_VERSION_TELINK_RELEASE                (VERSION_GET(Major, Minor, Revision))	//it will set to firmware_revision[6] in user_init.


#define PID_UNKNOW              (0x0000)

// ------ ------ Mesh PID ------ ------
// ------ light ------
// LIGHT_TYPE_SEL_              (0x00XX)
// ------ gateway ------
#define PID_GATEWAY             (0x0101)
// ------ LPN ------
#define PID_LPN                 (0x0201)
// ------ SWITCH ------
#define PID_SWITCH              (0x0301)
// ------ SPIRIT_LPN ------
#define PID_SPIRIT_LPN          (0x0401)


// ------ ------ Homekit PID ------ ------

#define PID_HK_NONMFI           (0xC001)

#define PID_HK_AC20C            (0xC002)

#define PID_HK_AC30             (0xC003)

#define PID_HK_SOFTAUTH         (0xC004)

/*
 * HOMEKIT_PID_SEL : PID is product ID,
 * HOMEKIT_VID: VID is software version ID,
 */

#if (MFI_ENABLE == 0x22)
#define HOMEKIT_PID_SEL		(PID_HK_AC20C)
#define HOMEKIT_VID		    (FW_VERSION_TELINK_RELEASE)       // user can redefine
#elif (MFI_ENABLE == 0x20)
#define HOMEKIT_PID_SEL		(PID_HK_AC30)
#define HOMEKIT_VID		    (FW_VERSION_TELINK_RELEASE)       // user can redefine
#else
#if SOFTWARE_AUTHENTICATION_ENABLE
#define HOMEKIT_PID_SEL		(PID_HK_SOFTAUTH)
#define HOMEKIT_VID		    (FW_VERSION_TELINK_RELEASE)       // user can redefine
#else
#define HOMEKIT_PID_SEL		(PID_HK_NONMFI)
#define HOMEKIT_VID		    (FW_VERSION_TELINK_RELEASE)       // user can redefine
#endif
#endif

#define BUILD_VERSION		(HOMEKIT_PID_SEL|(HOMEKIT_VID << 16))	// if value change, must make clean. same sequence with cps

/* Disable C linkage for C++ Compilers: */
#if defined(__cplusplus)
}
#endif
