/********************************************************************************************************
 * @file     app.c
 *
 * @brief    for TLSR chips
 *
 * @author	 public@telink-semi.com;
 * @date     Sep. 18, 2018
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/


#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"

#include "vendor/common/blt_led.h"
#include "vendor/common/blt_common.h"
#include "application/keyboard/keyboard.h"
#include "application/usbstd/usbkeycode.h"

#include "common.h"
#include "hk_TopUUID.h"
#include "hk_CategoryDef.h"
#include "homekit_src/hk_adv_packet.h"
#include "homekit_src/hk_indicate_queue.h"
#include "homekit_src/hk_l2cap_handle.h"
#include "homekit_src/hk_att_extend.h"

#include "./mfi/mfi.h"
#include "./ota_vendor_cmd/ota_vendor_cmd.h"
#include "./led_event/led_ctrl.h"
#include "../../homekit_src/hk_ble_ll_ota.h"
#include "app_att.h"
#include "tc_simulate/hk_tc_simulate.h"
#include "bin_name_key_setting.h"
#include "battery_check/battery_check.h"
#include "../common/blt_soft_timer.h"
#include "../common/version.h"


#define 	ADV_IDLE_ENTER_DEEP_TIME			60  //60 s
#define 	CONN_IDLE_ENTER_DEEP_TIME			60  //60 s

#define 	MY_DIRECT_ADV_TMIE					2000000


#define     MY_APP_ADV_CHANNEL					BLT_ENABLE_ADV_37
#define 	MY_ADV_INTERVAL_MIN					ADV_INTERVAL_20MS
#define 	MY_ADV_INTERVAL_MAX					ADV_INTERVAL_30MS
#define 	MY_DISCONN_EVT_ADV_INTERVAL_MIN		ADV_INTERVAL_20MS
#define 	MY_DISCONN_EVT_ADV_INTERVAL_MAX		ADV_INTERVAL_20MS

#define		MY_RF_POWER_INDEX					RF_POWER_P3p01dBm


#define		BLE_DEVICE_ADDRESS_TYPE 			BLE_DEVICE_ADDRESS_RANDOM_STATIC

_attribute_data_retention_	own_addr_type_t 	app_own_address_type = OWN_ADDRESS_RANDOM;


#define RX_FIFO_SIZE	64
#define RX_FIFO_NUM		8

#define TX_FIFO_SIZE	40
#define TX_FIFO_NUM		16


#if 0
	MYFIFO_INIT(blt_rxfifo, RX_FIFO_SIZE, RX_FIFO_NUM);
#else
_attribute_data_retention_  u8 		 	blt_rxfifo_b[RX_FIFO_SIZE * RX_FIFO_NUM] = {0};
_attribute_data_retention_	my_fifo_t	blt_rxfifo = {
												RX_FIFO_SIZE,
												RX_FIFO_NUM,
												0,
												0,
												blt_rxfifo_b,};
#endif


#if 0
	MYFIFO_INIT(blt_txfifo, TX_FIFO_SIZE, TX_FIFO_NUM);
#else
	_attribute_data_retention_  u8 		 	blt_txfifo_b[TX_FIFO_SIZE * TX_FIFO_NUM] = {0};
	_attribute_data_retention_	my_fifo_t	blt_txfifo = {
													TX_FIFO_SIZE,
													TX_FIFO_NUM,
													0,
													0,
													blt_txfifo_b,};
#endif

/************************************************************************
*		    Variables Definitions/Declarations
 ***********************************************************************/
_attribute_data_retention_	u32 broadcast_encryption_time_timeout = 0;
_attribute_data_retention_	u32 disconnect_event_time_timeout = 0;
_attribute_data_retention_ extern u8 blt_adv_global_state_modify_once;

_attribute_data_retention_	extern u8 EventNotifyCNT;

_attribute_data_retention_	u32 store_char_change_value_timeout= 0;

_attribute_data_retention_	u32 led_event_pending;

_attribute_data_retention_ volatile	u32 flash_char_value_addr = 0;

_attribute_data_retention_	store_char_change_value_t *char_save_restore = 0;

_attribute_data_retention_ u8 ReCheckADVStatusFlag = 1;

_attribute_data_retention_ u8 ReCheckAndRestoreFlashCharValueFlag = 0;

_attribute_data_retention_ u8 DoBroadcastEncrypDataCALL_flag = 0;
_attribute_data_retention_ u32 remove_invalid_srpkey_timeout = 0;

_attribute_data_retention_	u32	lowBattDet_tick   = 0;

_attribute_data_retention_ u32 adv_blt_soft_timeout = 0;
_attribute_data_retention_ u8 app_adv_state = 0;

extern st_ll_conn_slave_t		bltc;

extern void hkble_ota_set_flag(void);
extern void start_reboot(void);
extern int factory_reset_cnt_check ();

extern u8 hkble_ota_data_check;

extern u32 session_timeout_tick;
extern u32 hap_transationID_valid_timeout;
extern u32 hap_timed_write_timeout;
extern u32 hap_procedure_valid_timeout;
extern u32 hap_remove_pairing_timeout;

#if (FLASH_ADR_TYPE == FLASH_ADR_TYPE_DUAL_MODE)

// ------------------------OTA authorization code
/* APP: write "OTA start  + auth_value[16]" --> slave: check auth value --> 
** APP: read                                            --> slave: read response with ERR code(success:0, fail: 1)
  */

u8	dualmode_ota_key_user[8] = {0x01,0x02,0x03,0x04,'t','l','n','k',};
#endif

/************************************************************************
*	 Adv Packet, Response Packet
 ***********************************************************************/
const u8	tbl_advData[] = {
	0x05, 0x09, 't', 'H', 'I', 'D',
	0x02, 0x01, 0x05, 							// BLE limited discoverable mode and BR/EDR not supported
	0x03, 0x19, 0x80, 0x01, 					// 384, Generic Remote Control, Generic category
	0x05, 0x02, 0x12, 0x18, 0x0F, 0x18,		// incomplete list of service class UUIDs (0x1812, 0x180F)
};

// homekit scan rsp
const u8 tbl_rsp[] = { 0x0, 6+31, 0xef, 0xe1, 0xe2, 0xe3, 0xe4, 0xe5,	\
	                    0x1e,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};

_attribute_data_retention_	u8	tbl_mac[] = { 0xA0, 0xA1, 0xA2, 0xA3, 0xA4, 0xA5 };

_attribute_data_retention_	int device_in_connection_state;

_attribute_data_retention_	u32 advertise_begin_tick;

_attribute_data_retention_	u32	interval_update_tick;

_attribute_data_retention_	u8	sendTerminate_before_enterDeep = 0;

_attribute_data_retention_	u32	latest_user_event_tick;

/************************************************************************
 * 					procedure
 ***********************************************************************/
#if ENCODE_OTA_BIN_EN
void app_data_aes_encode(void * p, u8 len)
{
	u8 dat[16] = {0};
	extern const u8 key_encode_bin[];
	extern int aes_decrypt(unsigned char *Key, unsigned char *Data, unsigned char *Result);
	aes_decrypt((unsigned char *)key_encode_bin, p, dat);

	memcpy(p, dat, len);
}
#else
void app_data_aes_encode(void * p, u8 len){}
#endif

#if OTA_FW_TYPE_VER_CHECK_EN
unsigned char hkble_ota_FwTypeVerCheck(void * p, u8 len)
{
	u8 *dat = (u8 *)p;
	unsigned char err_flag = 0;

	u16 product_id = 0;
	u16 sw_version_id_new = 0;

	u8 ver_l = (FW_VERSION_TELINK_RELEASE >> 0) & 0xff;
	u8 ver_h = (FW_VERSION_TELINK_RELEASE >> 8) & 0xff;
	u16 sw_version_id_old = (ver_l << 8) | ver_h;

	product_id = (*dat) | (*(dat+1) << 8);
	sw_version_id_new = (*(dat+2) << 8) | (*(dat+3));

	if((product_id != PID_HK_NONMFI) && (product_id != PID_HK_AC20C) && (product_id != PID_HK_AC30) && (product_id != PID_HK_SOFTAUTH))
	{
		err_flag = HKBLE_OTA_FW_TYPE_ERR;
	}
	else if(sw_version_id_new <= sw_version_id_old)
	{
		err_flag = HKBLE_OTA_FW_VER_ERR;
	}
	
	return err_flag;
}
#endif

//for assert the validity of Macro(Major, Minor, Revision) at compile time.
STATIC_ASSERT(Major <= 9);
STATIC_ASSERT(Minor <= 99);
STATIC_ASSERT(Revision <= 9);

void dev_firmware_revision_get(void)
{
	u8 ver_str[6] = {0};
	u8 ver_l = (FW_VERSION_TELINK_RELEASE >> 0) & 0xff;
	u8 ver_h = (FW_VERSION_TELINK_RELEASE >> 8) & 0xff;

	ver_str[0] = (ver_l >> 4) + '0';
	ver_str[1] = '.';
	ver_str[2] = ((ver_l >> 0) & 0x0f) + '0';
	ver_str[3] = ((ver_h >> 4) & 0x0f) + '0';
	ver_str[4] = '.';
	ver_str[5] = ((ver_h >> 0) & 0x0f) + '0';

	memcpy(firmware_revision, ver_str, sizeof(firmware_revision));
}

void cfg_led_event(u32 e)
{
    led_event_pending = e;
}

/************************************************************************
 * 					LUM ADJUST
 ***********************************************************************/

void rf_link_light_event_callback (u8 status)
{
    if(status == 0xC5)
    {
#if LIGHTBULB_SERVICE_ENABLE
        u8 cur_stat = light_onoff;
        foreach(i, 6){
            light_enable (cur_stat ^ (i&0x01));
            sleep_us(200 * 1000);
        }
        light_enable (cur_stat);
#endif
    }
}

_USER_CAN_REDEFINE_ void light_enable (int en)
{
	#if LIGHTBULB_SERVICE_ENABLE
    light_onoff = en;
	#endif
    pwm_light_enable (en);

////////////////////////////////////
// It should be removed here due to those services are independent. by Andrew Lee
#if LOCKMECHANISM_SERVICE_ENABLE
#if DOORLOCK_LED_DEMO_ENABLE
    if(Lock_Target_State)
        pwm_set_cycle_and_duty(PWMID_W, PMW_MAX_TICK, PMW_MAX_TICK);
    else
        pwm_set_cycle_and_duty(PWMID_W, PMW_MAX_TICK, 0);
#endif
#endif
}


//save characteristic value, if the value was updated for a while
u8 dev_characteristic_store_to_flash(void){

    store_char_change_value_t char_save_store;

    flash_char_value_addr += sizeof(store_char_change_value_t);

    if((flash_char_value_addr >= ((u32)flash_adr_layout.flash_adr_char_value + FLASH_SECTOR_SIZE - sizeof(store_char_change_value_t)))){
        flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_char_value);
        flash_char_value_addr = (u32)flash_adr_layout.flash_adr_char_value;
    }else{
        flash_char_value_addr -= sizeof(store_char_change_value_t);
        u8 clear_save_flag = 0;
        flash_write_page(flash_char_value_addr, 1, (u8 *)(&clear_save_flag));
        flash_char_value_addr += sizeof(store_char_change_value_t);
    }

    STORE_CHAR_TO_FLASH

    flash_write_page(flash_char_value_addr, sizeof(char_save_store), (u8 *)(&char_save_store));

    return 1;
}


void dev_char_val_update_from_flash(void)
{
	int i;
	unsigned char do_effect_flag = 0;

	for(i = 0; i < (FLASH_SECTOR_SIZE); i += sizeof(store_char_change_value_t)){
		flash_char_value_addr = (u32)flash_adr_layout.flash_adr_char_value + i;

		char_save_restore = (store_char_change_value_t *)(flash_char_value_addr);
		if(0 == char_save_restore->save_flag){
			continue;
		}
		else if(CHAR_SAVE_FLAG == char_save_restore->save_flag)
		{
			RESTORE_CHAR_FROM_FLASH
			do_effect_flag = 1;
			break;
		}
//- 	else if(0x1F != char_save_restore->save_flag)
//- 	{
//- 		//invalid
//- 		continue;
//- 	}
		else
		{
			break;
		}
	}


	if(do_effect_flag == 0)
	{
		dev_characteristic_store_to_flash();
	}
}


//retrieve device characteristic value from flash
void dev_char_val_restore_from_flash(void){

	dev_char_val_update_from_flash();

    //effect
//--    if(do_effect_flag)
//--    {
//--        do_effect_flag = 0;
        /////////////////////////////////
        // do effect action here!
        /////////////////////////////////
#if LIGHTBULB_SERVICE_ENABLE
        extern void Convert_HSV2RGB(u32 * dbg);
        u32 dbg2 = 0;
        Convert_HSV2RGB(&dbg2);
        light_enable (light_onoff);
#endif

#if LOCKMECHANISM_SERVICE_ENABLE
        Doorlock_Proc_Flg = 1;
        //-doorlock_proc();
#endif
//--    }
}

void assign_current_flash_char_value_addr(void)
{
    volatile u32 temp_flash_char_value_addr = (u32)flash_adr_layout.flash_adr_char_value;

    int i;
    for(i = 0; i < (FLASH_SECTOR_SIZE); i += sizeof(store_char_change_value_t)) {
        temp_flash_char_value_addr = (u32)flash_adr_layout.flash_adr_char_value + i;

        char_save_restore = (store_char_change_value_t *)(temp_flash_char_value_addr);
        if (0 == char_save_restore->save_flag){
            continue;
        } else if (CHAR_SAVE_FLAG == char_save_restore->save_flag) {
            #if 0   // for dbg
            static volatile store_char_change_value_t* light_brightness_addr = (store_char_change_value_t*) flash_adr_layout.flash_adr_char_value;
            light_brightness_addr = char_save_restore;
            #endif

            break;
//-        } else if (0x1F != char_save_restore->save_flag) {
//-            //invalid
//-            continue;
        } else {
            break;
        }
    }
}

void store_char_value_to_flash(void)
{
    store_char_change_value_timeout = clock_time()|1;
}

/************************************************************************
 * 					WAKEUP SOURCE SET
 ***********************************************************************/



/************************************************************************
 * 					UI,LED, PROC TASK
 ***********************************************************************/
void proc_ui(void) {

    static u32 btn_sw2_pre = 1;
	static u32 btn_sw3_pre = 1;
	static u32 btn_sw4_pre = 1;
	static u32 btn_sw5_pre = 1;
	u32 b1 = 0;
	u32 b2 = 0;

	#if LIGHTBULB_SERVICE_ENABLE
	u8 gsn_modify_temp = blt_adv_global_state_modify_once;	//just implement lightblue service test.
	#endif
	
	gpio_write(BUTTON_SW2_SW3_BASE, 0);
	gpio_write(BUTTON_SW4_SW5_BASE, 1);
	sleep_us(1);
    b1 = gpio_read(BUTTON_SW2_SW4_PIN);		//SW2
	b2 = gpio_read(BUTTON_SW3_SW5_PIN);		//SW3
	
    if (!b1 && btn_sw2_pre) {

		#if (BLT_SOFTWARE_TIMER_ENABLE)
		if(app_adv_state && (!pairPaired())){
			app_adv_state = 0;
			bls_ll_setAdvEnable(1);  //adv enable
			blt_soft_timer_tick_start();
			adv_blt_soft_timeout = blt_soft_timer_clock_time();
		}
		#endif
        /////////////////////////////////
        // do key1 action here!
        /////////////////////////////////
#if LIGHTBULB_SERVICE_ENABLE

		light_onoff = !light_onoff;
        store_char_value_to_flash();
		
        if(EventNotifyCNT)
            if(bltc.conn_terminate_pending == 0)
            {
                if(indicate_handle_no_send_once_after_checkit(LIGHTBULB_HANDLE_NUM_LIGHT_ONOFF) == 1)
                    sleep_us(250);
            }
			
        blt_adv_set_global_state ();
		if( blt_state == BLS_LINK_STATE_ADV)
		{
			#if 1
			if(bc_para_light_on & BIT(7)){
			blt_broadcast_set_global_state();
			DoBroadcastEncrypDataCALL_flag++;
			}else if(gsn_modify_temp){
				bls_ll_setAdvInterval(MY_DISCONN_EVT_ADV_INTERVAL_MIN, MY_DISCONN_EVT_ADV_INTERVAL_MAX); // disconnect event adv interval = 20ms
				disconnect_event_time_timeout = clock_time() | 1;
			}
			#endif
		}
    	
    	pwm_light_enable (light_onoff);
#endif  //for LIGHTBULB_SERVICE_ENABLE action

#if DBG_UART_SHOW_APPEND_PROC_TIME
        uart_proc_time_max = 0;
#endif

    }
    btn_sw2_pre = b1;

    if (!b2 && btn_sw3_pre) {

#if (LIGHTBULB_SERVICE_ENABLE && CHAR_BRIGHTNESS_ENABLE)

		if(light_brightness != 0)
        	light_brightness = 0;
		else
			light_brightness = 90;
        store_char_value_to_flash();
		
        if(EventNotifyCNT)
            if(bltc.conn_terminate_pending == 0)
            {
                if(indicate_handle_no_send_once_after_checkit(LIGHTBULB_HANDLE_NUM_LIGHT_BRIGHTNESS) == 1)
                    sleep_us(250);
            }
			
        blt_adv_set_global_state ();
		if( blt_state == BLS_LINK_STATE_ADV)
		{
	#if 1
			if(bc_para_light_bri & BIT(7)){
			blt_broadcast_set_global_state();
			DoBroadcastEncrypDataCALL_flag++;
			}else if(gsn_modify_temp){
				bls_ll_setAdvInterval(MY_DISCONN_EVT_ADV_INTERVAL_MIN, MY_DISCONN_EVT_ADV_INTERVAL_MAX); // disconnect event adv interval = 20ms
				disconnect_event_time_timeout = clock_time() | 1;
			}
	#endif
		}
		
		extern void Convert_HSV2RGB(u32 * dbg);
		u32 dbg2 = 0;
		Convert_HSV2RGB(&dbg2);
#endif  //for LIGHTBULB_SERVICE_ENABLE action
    }
    btn_sw3_pre = b2;


	gpio_write(BUTTON_SW2_SW3_BASE, 1);
	gpio_write(BUTTON_SW4_SW5_BASE, 0);
	sleep_us(1);
    b1 = gpio_read(BUTTON_SW2_SW4_PIN);		//SW4
	b2 = gpio_read(BUTTON_SW3_SW5_PIN);		//SW5

    if (!b1 && btn_sw4_pre) {

#if (LIGHTBULB_SERVICE_ENABLE && CHAR_HUE_ENABLE)

		if(hue_value != 0)
        	hue_value = 0;
		else
			hue_value = 0x42C80000;	//100.0
        store_char_value_to_flash();
		
        if(EventNotifyCNT)
            if(bltc.conn_terminate_pending == 0)
            {
                if(indicate_handle_no_send_once_after_checkit(LIGHTBULB_HANDLE_NUM_HUE_VALUE) == 1)
                    sleep_us(250);
            }
        blt_adv_set_global_state ();
			
		if( blt_state == BLS_LINK_STATE_ADV)
		{
			#if 1
			if(bc_para_light_hue & BIT(7)){
			blt_broadcast_set_global_state();
			DoBroadcastEncrypDataCALL_flag++;
			}else if(gsn_modify_temp){
				bls_ll_setAdvInterval(MY_DISCONN_EVT_ADV_INTERVAL_MIN, MY_DISCONN_EVT_ADV_INTERVAL_MAX); // disconnect event adv interval = 20ms
				disconnect_event_time_timeout = clock_time() | 1;
			}
			#endif
		}
			
		extern void Convert_HSV2RGB(u32 * dbg);
		u32 dbg2 = 0;
		Convert_HSV2RGB(&dbg2);
#endif  //for LIGHTBULB_SERVICE_ENABLE action

    }
    btn_sw4_pre = b1;

    if (!b2 && btn_sw5_pre) {

#if (LIGHTBULB_SERVICE_ENABLE && CHAR_SATURATION_ENABLE)

		if(saturation_value != 0)
        	saturation_value = 0;
		else
			saturation_value = 0x42480000;	//50.0
        store_char_value_to_flash();
		
        if(EventNotifyCNT)
            if(bltc.conn_terminate_pending == 0)
            {
                if(indicate_handle_no_send_once_after_checkit(LIGHTBULB_HANDLE_NUM_SATURATION_VALUE) == 1)
                    sleep_us(250);
            }
			
        blt_adv_set_global_state ();
		if( blt_state == BLS_LINK_STATE_ADV)
		{
			#if 1
			if(bc_para_light_satu & BIT(7)){
			blt_broadcast_set_global_state();
			DoBroadcastEncrypDataCALL_flag++;
			}else if(gsn_modify_temp){
				bls_ll_setAdvInterval(MY_DISCONN_EVT_ADV_INTERVAL_MIN, MY_DISCONN_EVT_ADV_INTERVAL_MAX); // disconnect event adv interval = 20ms
				disconnect_event_time_timeout = clock_time() | 1;
			}
			#endif
		}
			
		extern void Convert_HSV2RGB(u32 * dbg);
		u32 dbg2 = 0;
		Convert_HSV2RGB(&dbg2);
#endif  //for LIGHTBULB_SERVICE_ENABLE action
    }
    btn_sw5_pre = b2;
}


void proc_led(void) {

    static unsigned int led_ton;
    static unsigned int led_toff;
    static unsigned char led_count = 0;
    static unsigned char led_sel;
    static u32 led_last_tick;
    static unsigned char led_last_indicate_state;
    unsigned int led_apply_duty_count;

    if (!led_count && !led_event_pending)
        return;  //led flash finished

    if (led_event_pending)      // update led event parameters
    {
        // new event
        led_ton = (led_event_pending & 0xFF) * 5 * CLOCK_SYS_CLOCK_1MS;
        led_toff = ((led_event_pending >> 8) & 0xFF) * 5 * CLOCK_SYS_CLOCK_1MS;
        led_count = (led_event_pending >> 16) & 0xFF;
        led_sel = led_event_pending >> 24;

        led_event_pending = 0;
        led_last_tick = clock_time() + 30 * CLOCK_SYS_CLOCK_1S;
        led_last_indicate_state = 0;
    }

    if(led_last_indicate_state)
        led_apply_duty_count = led_ton;
    else
        led_apply_duty_count = led_toff;

    if ((u32) (clock_time() - led_last_tick) >= led_apply_duty_count)
    {
        led_last_tick = clock_time();
        led_count--;
        led_last_indicate_state ^= 1;

        if(led_count == 0)
        {
            #if LIGHTBULB_SERVICE_ENABLE
                    extern void Convert_HSV2RGB(u32 * dbg);
                    u32 dbg2 = 0;
                    Convert_HSV2RGB(&dbg2);
                    light_enable (light_onoff);
            #endif
            //-pwm_light_enable(light_onoff_status);   // recover to original LED status
            return;
        }

        if (led_last_indicate_state)
        {
            if (led_sel == 0x0F)
            {
                pwm_set_lum(PWMID_G, LED_INDICATE_LUM_VAL, 0);
                pwm_set_lum(PWMID_B, LED_INDICATE_LUM_VAL, 0);
                pwm_set_lum(PWMID_R, LED_INDICATE_LUM_VAL, 0);
                pwm_set_lum(PWMID_W, LED_INDICATE_LUM_VAL, 0);
            }
            else if (led_sel == 0x07)
            {
                pwm_set_lum(PWMID_G, LED_INDICATE_LUM_VAL, 0);
                pwm_set_lum(PWMID_B, LED_INDICATE_LUM_VAL, 0);
                pwm_set_lum(PWMID_R, LED_INDICATE_LUM_VAL, 0);
            }
            else
            {
                if (led_sel & BIT(0)) pwm_set_lum(PWMID_G, LED_INDICATE_LUM_VAL, 0);
                if (led_sel & BIT(1)) pwm_set_lum(PWMID_B, LED_INDICATE_LUM_VAL, 0);
                if (led_sel & BIT(2)) pwm_set_lum(PWMID_R, LED_INDICATE_LUM_VAL, 0);
                if (led_sel & BIT(3)) pwm_set_lum(PWMID_W, LED_INDICATE_LUM_VAL, 0);
            }
        }
        else
        {
            if (led_sel == 0x0F)
            {
                pwm_set_lum(PWMID_G, 0, 0);
                pwm_set_lum(PWMID_B, 0, 0);
                pwm_set_lum(PWMID_R, 0, 0);
                pwm_set_lum(PWMID_W, 0, 0);
            }
            else if (led_sel == 0x07)
            {
                pwm_set_lum(PWMID_G, 0, 0);
                pwm_set_lum(PWMID_B, 0, 0);
                pwm_set_lum(PWMID_R, 0, 0);
            }
            else
            {
                if (led_sel & BIT(0)) pwm_set_lum(PWMID_G, 0, 0);
                if (led_sel & BIT(1)) pwm_set_lum(PWMID_B, 0, 0);
                if (led_sel & BIT(2)) pwm_set_lum(PWMID_R, 0, 0);
                if (led_sel & BIT(3)) pwm_set_lum(PWMID_W, 0, 0);
            }
        }
    }
}

void set_broadcast_adv_interval(u8 bc_param)
{
	u16 interval = 0;
	enum{
		BROADCAST_INTERVAL_20MS			= 0x01,
		BROADCAST_INTERVAL_1280MS		= 0x02,
		BROADCAST_INTERVAL_2560MS		= 0x03,
	};
	
	bc_param &= ~(BIT(7));
	
	if(bc_param == BROADCAST_INTERVAL_20MS)
	{
		interval = ADV_INTERVAL_20MS;
	}
	else if(bc_param == BROADCAST_INTERVAL_1280MS)
	{
		interval = ADV_INTERVAL_1_28_S;
	}
	else if(bc_param == BROADCAST_INTERVAL_2560MS)
	{
		interval = ADV_INTERVAL_2_56_S;
	}
	else
	{
		interval = ADV_INTERVAL_20MS;	//default value.
	}

	bls_ll_setAdvInterval(interval, interval);
}

void proc_broadcast_event()
{

	if( broadcast_encryption_time_timeout && clock_time_exceed(broadcast_encryption_time_timeout,3*1100*1000) ){
		broadcast_encryption_time_timeout = 0;

		//	TODO:  reset adv packet to normal adv data.
		blt_set_normal_adv_pkt(HK_CATEGORY, DEVICE_SHORT_NAME);
		bls_ll_setAdvInterval(MY_DISCONN_EVT_ADV_INTERVAL_MIN, MY_DISCONN_EVT_ADV_INTERVAL_MAX); // disconnect event adv interval = 20ms
		disconnect_event_time_timeout = clock_time() | 1;			
	}

	if(disconnect_event_time_timeout && clock_time_exceed(disconnect_event_time_timeout, 3*1100*1000)){
		disconnect_event_time_timeout = 0;
		bls_ll_setAdvInterval(MY_ADV_INTERVAL_MIN, MY_ADV_INTERVAL_MAX);
	}
}

/************************************************************************
*							MAC  ADDR  SET
 ***********************************************************************/
/*user should update the mac addr after firmware update.*/
void dev_mac_addr_update(void)
{
	u8	mac_random_static[6];
	
#if (FLASH_ADR_TYPE == FLASH_ADR_TYPE_DEFAULT)
	flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_mac);
	
	/*mac addr generate by randomly*/
	blc_initMacAddress(CFG_ADR_MAC, tbl_mac, mac_random_static);
#elif (FLASH_ADR_TYPE == FLASH_ADR_TYPE_DUAL_MODE)
	/*clear other mac addr of homekit*/
	dev_random_static_mac_addr_set((unsigned long)flash_adr_layout.flash_adr_mac, true, tbl_mac, mac_random_static);
#endif

#if(BLE_DEVICE_ADDRESS_TYPE == BLE_DEVICE_ADDRESS_PUBLIC)
		app_own_address_type = OWN_ADDRESS_PUBLIC;
#elif(BLE_DEVICE_ADDRESS_TYPE == BLE_DEVICE_ADDRESS_RANDOM_STATIC)
		app_own_address_type = OWN_ADDRESS_RANDOM;
		blc_ll_setRandomAddr(mac_random_static);
#endif

}

#if (FLASH_ADR_TYPE == FLASH_ADR_TYPE_DUAL_MODE)
void dev_flash_adr_table_init(void)
{
	u32 fw_flag = 0;
	flash_read_page((unsigned long)flash_adr_layout.flash_adr_firmware_type, 4, (u8 *)&fw_flag);
	
	if(fw_flag != TYPE_TLK_HOMEKIT)
	{
		/*set firmware type of homekit.*/
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_firmware_type);
		fw_flag = TYPE_TLK_HOMEKIT;
		flash_write_page ((unsigned long)flash_adr_layout.flash_adr_firmware_type, 4, (u8 *)&fw_flag);

		/*clear other mac addr of homekit*/
		u8 mac[FLASH_ADR_MAC_RETAINED_LEN];
		flash_read_page((unsigned long)flash_adr_layout.flash_adr_mac, FLASH_ADR_MAC_RETAINED_LEN, mac);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_mac);
		flash_write_page ((unsigned long)flash_adr_layout.flash_adr_mac, FLASH_ADR_MAC_RETAINED_LEN, mac);

		/*erase flash for homekit sdk data.*/
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_char_value);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_device_id);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_hash_id);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_accessory_ltsk);
		//flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_srp_key);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_id_info);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_id_info + 0x1000);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_factory_reset_cnt);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_global_state);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_fw_update_cnt);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_adv_encryption_key);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_new_id);
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_broadcast_gsn);
		//flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_srp_key2);
		
	}
	return;
}
#endif
/************************************************************************
*						SETUP	CODE 	SET
 ***********************************************************************/
void dev_setup_code_set(void)
{
#if SETUP_CODE_RANDOM_ENABLE

    volatile unsigned char * p_setup_code = (flash_adr_layout.flash_adr_setupcode);

    if((p_setup_code[0]==0xFF)&&(p_setup_code[1]==0xFF)&&(p_setup_code[2]==0xFF)&&(p_setup_code[3]==0xFF))
    {
        u8 bits;
        u16 temp_data1 = myrand();
        u16 temp_data2 = myrand();
        bool setup_code_valid = true;

        foreach(i, 4)
        {
            bits = ((temp_data1 >> (i * 4)) & 0x0F);
            if(bits >= 0x0A)
            {
                temp_data1 &= ~(0x0F << (i * 4));
                temp_data1 |= ((bits - 0x0A) << (i * 4));
            }
            bits = ((temp_data2 >> (i * 4)) & 0x0F);
            if(bits >= 0x0A)
            {
                temp_data2 &= ~(0x0F << (i * 4));
                temp_data2 |= ((bits - 0x0A) << (i * 4));
            }
        }

        /* verify the randomly generated setup code if match with the request of HAP or not. */
        if(temp_data1 == temp_data2)
        {
            if(((temp_data1 >> 0) & 0xFF) == ((temp_data1 >> 8) & 0xFF))
                if(((temp_data1 >> 0) & 0x0F) == ((temp_data1 >> 4) & 0x0F))
                    setup_code_valid = false;
        }
        else
        {
            if(((((temp_data1 >> 0) & 0xFF) == 0x21) && (((temp_data1 >> 8) & 0xFF) == 0x43) && \
                (((temp_data2 >> 0) & 0xFF) == 0x65) && (((temp_data2 >> 8) & 0xFF) == 0x87)) || \
                ((((temp_data1 >> 0) & 0xFF) == 0x78) && (((temp_data1 >> 8) & 0xFF) == 0x56) && \
                (((temp_data2 >> 0) & 0xFF) == 0x34) && (((temp_data2 >> 8) & 0xFF) == 0x12)))
            {
                setup_code_valid = false;
            }
        }

        if(!setup_code_valid)
        {
            temp_data1 += 1;

            bits = ((temp_data1 >> 0) & 0x0F);
            if(bits >= 0x0A)
            {
                temp_data1 &= ~(0x0F << 0);
                temp_data1 |= ((bits - 0x0A) << 0);
            }
        }

        SetupCodeData = ((temp_data1 << 0 ) | (temp_data2 << 16 ));
        flash_write_page((u32)p_setup_code, 4, (u8 *)&SetupCodeData);

    }
    else
    {
        memcpy ((u8 *)&SetupCodeData, p_setup_code, 4);

		#if (FLASH_ADR_TYPE == FLASH_ADR_TYPE_DEFAULT)
        u8 wr_cnts = 0;
        flash_read_page((unsigned long)flash_adr_layout.flash_adr_setupcode_write_cnt, 1, &wr_cnts);
        SetupCodeWriteCnts = ((wr_cnts == 0xFF) ? 0 : wr_cnts);

        SetupCodeReadCnts = flash_subregion_read_val((unsigned long)flash_adr_layout.flash_adr_setupcode_read_cnt, 0, SETUP_CODE_READ_CNTS_MAX);
		#endif
    }

#endif
}

/************************************************************************
*						SERIAL NUMBER SET
 ***********************************************************************/
void dev_serial_number_set(void)
{
#if SERIAL_NUMBER_RANDOM_ENABLE

    u32 *p_serial_num = (u32 *) flash_adr_layout.flash_adr_serial_number;

    if (*p_serial_num == 0xffffffff)
    {
        u8 byte;
        u8 sn_data;
        u16 data_array[5];

        /*0~9 : (0x30 - 0x39), A~Z : (0x41 - 0x5A), a~z : (0x61 - 0x7A)
          *serial number consist of Arabic numerals and uppercase and lowercase
          *that add up to 62(10 + 26 * 2).
          */

        foreach(i, 5)
        {
            data_array[i] = myrand();

            foreach(j, 2)
            {
                byte = ((data_array[i] >> (j * 8)) & 0xFF);
                sn_data = byte;

                if(byte < '0')
                {
                    sn_data = byte % 9 + '0';
                }
                else if((byte > '9') && (byte < 'A'))
                {
                    sn_data = byte % 9 + '0';
                }
                else if((byte > 'Z') && (byte < 'a'))
                {
                    sn_data = byte % 26 + 'A';
                }
                else if(byte > 'z')
                {
                    sn_data = byte % 26 + 'a';
                }
                //lowercase --> uppercase
                if((sn_data >= 'a') && (sn_data <= 'z'))
                {
                    sn_data -= 32;
                }

                data_array[i] &= ~(0xFF << (j * 8));
                data_array[i] |= (sn_data << (j * 8));
            }
            memcpy(serialNumber + (i * 2), (u8 *)&data_array[i], 2);
        }
        flash_write_page((unsigned long)flash_adr_layout.flash_adr_serial_number, sizeof(serialNumber), serialNumber);

    }
    else
    {
        flash_read_page((unsigned long)flash_adr_layout.flash_adr_serial_number, sizeof(serialNumber), serialNumber);
    }

#endif
}

/************************************************************************
*							SETUP ID SET
 ***********************************************************************/
void dev_setup_id_set(void)
{
#if SETUP_ID_RANDOM_ENABLE

    volatile unsigned char * p_setup_id = (flash_adr_layout.flash_adr_setup_id);

    if((p_setup_id[0]==0xFF)&&(p_setup_id[1]==0xFF)&&(p_setup_id[2]==0xFF)&&(p_setup_id[3]==0xFF))
    {
        u8 byte1;
        u8 setup_id_data;
        u16 data_array1[2];

        foreach(i, 2)
        {
            data_array1[i] = myrand();

            foreach(j, 2)
            {
                byte1 = ((data_array1[i] >> (j * 8)) & 0xFF);
                setup_id_data = byte1;

                if(byte1 < '0')
                {
                    setup_id_data = byte1 % 9 + '0';
                }
                else if((byte1 > '9') && (byte1 < 'A'))
                {
                    setup_id_data = byte1 % 9 + '0';
                }
                else if(byte1 > 'Z')
                {
                    setup_id_data = byte1 % 26 + 'A';
                }
                data_array1[i] &= ~(0xFF << (j * 8));
                data_array1[i] |= (setup_id_data << (j * 8));
            }
        }

        SetupIdData = ((data_array1[0] << 0) | (data_array1[1] << 16));
        flash_write_page((u32)p_setup_id, 4, (u8 *)&SetupIdData);
    }
    else
    {
        memcpy((u8 *)&SetupIdData, p_setup_id, 4);
    }

#endif
}

/************************************************************************
*						   SW_AUTHENTICATION ENABLE
 ***********************************************************************/
#if SOFTWARE_AUTHENTICATION_ENABLE
/*62DAD088-284B-4695-89FE-D650299BED25*/
u8 test_sw_uuid[] = {0x25, 0xED, 0x9B, 0x29, 0x50, 0xD6, 0xFE, 0x89,
						0x95, 0x46, 0x4B, 0x28, 0x88, 0xD0, 0xDA, 0x62};
/*
base64 encoded data:
MYGqME8CAQECAQEERzBFAiB3i5TIMOGrXaCoim+N1MVZ6mLrnqeCJr+mEGj7J7a8CwIhAKzkft/acMCFNt3ukeVxLa7rOxjXNJZPIM/CjQLyyvfXMFcCAQICAQEETzFNMAkCAWYCAQEEAQEwEAIBZQIBAQQIhJ8ro2oBAAAwFAICAMkCAQEECzI1MDAzMy0wMDM2MBgCAWcCAQEEEKJaREANe0qDls7VeN5gAX8=

base64 decoded data:
*/
u8 test_sw_token[] = {
0x31, 0x81, 0xAA, 0x30, 0x4F, 0x02, 0x01, 0x01, 0x02, 0x01, 0x01, 0x04, 0x47, 0x30, 0x45, 0x02,
0x20, 0x77, 0x8B, 0x94, 0xC8, 0x30, 0xE1, 0xAB, 0x5D, 0xA0, 0xA8, 0x8A, 0x6F, 0x8D, 0xD4, 0xC5,
0x59, 0xEA, 0x62, 0xEB, 0x9E, 0xA7, 0x82, 0x26, 0xBF, 0xA6, 0x10, 0x68, 0xFB, 0x27, 0xB6, 0xBC,
0x0B, 0x02, 0x21, 0x00, 0xAC, 0xE4, 0x7E, 0xDF, 0xDA, 0x70, 0xC0, 0x85, 0x36, 0xDD, 0xEE, 0x91,
0xE5, 0x71, 0x2D, 0xAE, 0xEB, 0x3B, 0x18, 0xD7, 0x34, 0x96, 0x4F, 0x20, 0xCF, 0xC2, 0x8D, 0x02,
0xF2, 0xCA, 0xF7, 0xD7, 0x30, 0x57, 0x02, 0x01, 0x02, 0x02, 0x01, 0x01, 0x04, 0x4F, 0x31, 0x4D,
0x30, 0x09, 0x02, 0x01, 0x66, 0x02, 0x01, 0x01, 0x04, 0x01, 0x01, 0x30, 0x10, 0x02, 0x01, 0x65,
0x02, 0x01, 0x01, 0x04, 0x08, 0x84, 0x9F, 0x2B, 0xA3, 0x6A, 0x01, 0x00, 0x00, 0x30, 0x14, 0x02,
0x02, 0x00, 0xC9, 0x02, 0x01, 0x01, 0x04, 0x0B, 0x32, 0x35, 0x30, 0x30, 0x33, 0x33, 0x2D, 0x30,
0x30, 0x33, 0x36, 0x30, 0x18, 0x02, 0x01, 0x67, 0x02, 0x01, 0x01, 0x04, 0x10, 0xA2, 0x5A, 0x44,
0x40, 0x0D, 0x7B, 0x4A, 0x83, 0x96, 0xCE, 0xD5, 0x78, 0xDE, 0x60, 0x01, 0x7F,
};

#endif

#if SOFTWARE_AUTHENTICATION_ENABLE
void dev_sw_auth_token_set(u8 set_flag, u8 * token_data, u16 token_len)
{

	bool store_to_flash = true;
	u16 flash_flag;

	if(set_flag == SW_AUTH_TOKEN_INIT)
	{
		flash_read_page((unsigned long)flash_adr_layout.flash_adr_sw_token, 2, (u8 *)&flash_flag);
		if(flash_flag == FLASH_FLAG_VALID)
		{
			store_to_flash = false;
		}
	}

	if(store_to_flash)
	{
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_sw_token);
		flash_write_page((unsigned long)flash_adr_layout.flash_adr_sw_token + 16, token_len, token_data);
		flash_flag = FLASH_FLAG_VALID;
		flash_write_page((unsigned long)flash_adr_layout.flash_adr_sw_token, 2, (u8 *)&flash_flag);
		flash_write_page((unsigned long)flash_adr_layout.flash_adr_sw_token + 2, 2, (u8 *)&token_len);
	}

	return;
}

void dev_sw_auth_uuid_set(u8 set_flag, u8 * auth_uuid)
{

	bool store_to_flash = true;
	u16 flash_flag;

	if(set_flag == SW_AUTH_UUID_INIT)
	{
		flash_read_page((unsigned long)flash_adr_layout.flash_adr_sw_uuid, 2, (u8 *)&flash_flag);
		if(flash_flag == FLASH_FLAG_VALID)
		{
			store_to_flash = false;
		}
	}

	if(store_to_flash)
	{
		flash_erase_sector((unsigned long)flash_adr_layout.flash_adr_sw_uuid);
		flash_write_page((unsigned long)flash_adr_layout.flash_adr_sw_uuid + 16, 16, auth_uuid);
		flash_flag = FLASH_FLAG_VALID;
		flash_write_page((unsigned long)flash_adr_layout.flash_adr_sw_uuid, 2, (u8 *)&flash_flag);
	}

	return;
}
#endif

/************************************************************************
 *  						callback function.
 ***********************************************************************/

int task_adv_pre(rf_packet_adv_t * p)		// called before adv packet sent.
{
    u8 pairPaired_flag = 0;

    if (ReCheckADVStatusFlag == 1)
    {
        ReCheckADVStatusFlag = 0;
        pairPaired_flag = pairPaired();
        u8 paired_status_flag_tmp;
        if (pairPaired_flag)
        {
            paired_status_flag_tmp = 1;
            blt_adv_update_pkt(ADV_PKT_PAIR_STATUS_FLAG, &paired_status_flag_tmp);
        }
        else
        {
            paired_status_flag_tmp = 0;
            blt_adv_update_pkt(ADV_PKT_PAIR_STATUS_FLAG, &paired_status_flag_tmp);
        }
#if NTC_TEMP_ENABLE
        extern u8 cnt_run_fNtcTemperCheck;
        cnt_run_fNtcTemperCheck= 0; // trigger ntc reset
#endif
    }

	return 1;				// must return 1.
}

#if 0
int task_adv_post(rf_packet_adv_t * p)
{

    proc_ui ();     // lightweight proc
    proc_led();     // lightweight proc
    proc_broadcast_event();

	return 1;
}

void task_connect_brx(void)
{
#if NTC_TEMP_ENABLE
        extern u8 cnt_run_fNtcTemperCheck;
        cnt_run_fNtcTemperCheck= 0; // trigger ntc reset
#endif
    proc_ui ();     // lightweight proc
    proc_led();     // lightweight proc
    proc_broadcast_event();
}
#endif

void task_ui()
{
	#if HK_TEST_CASE_SIMULATE_ENABLE
	proc_ui_simulate();
	#else
	proc_ui();
	#endif
	proc_led();
	proc_broadcast_event();
}

/************************************************************************
 *  					Led callback function.
 ***********************************************************************/
// Begin1stBootInitKeyPhase hook customize function
void Begin1stBootInitKeyPhaseCB_pwm_hw_flash_on_cfg_1HZ(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_1HZ);
}

// Finish1stBootInitKeyPhase hook customize function
void Finish1stBootInitKeyPhaseCB_pwm_hw_flash_off(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_STOP);
}

// ReceivePairRequestPhase hook customize function
void ReceivePairRequestPhaseCB(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_5HZ);
}

// WaitSetupCodeInPhase hook customize function
void WaitSetupCodeInPhaseCB(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_1HZ);
}

// FinishSetupCodeInOrTripleTryPhase hook customize function
void FinishSetupCodeInOrTripleTryPhaseCB(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_2HZ_3_SECOND);
}

// Register1stAdminKeyPhase hook customize function
void Register1stAdminKeyPhaseCB(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_4HZ);
    sleep_us(1000);                         // delay for flash operation
    cfg_led_event(LED_EVENT_FLASH_2HZ_2TIMES);
}

// DoRemovePair1Phase hook customize function
void DoRemovePair1PhaseCB(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_5HZ);
    sleep_us(1000);                         // delay for flash operation
    cfg_led_event(LED_EVENT_FLASH_2HZ_2TIMES);
}

// DoRemovePair2Phase hook customize function
void DoRemovePair2PhaseCB(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_5HZ);
    extern u8 ReCheckADVStatusFlag;
    ReCheckADVStatusFlag = 1;
    sleep_us(1000);                         // delay for flash operation
    cfg_led_event(LED_EVENT_FLASH_2HZ_5TIMES);
}

// DoRemoveAllPairPhase hook customize function
void DoRemoveAllPairPhaseCB(unsigned char HookPointNum, unsigned char * parameter) {
    cfg_led_event(LED_EVENT_FLASH_5HZ);
    extern u8 ReCheckADVStatusFlag;
    ReCheckADVStatusFlag = 1;
    // trigger re-check and restore characterstic value, if empty, it'll auto re-initial.
    ReCheckAndRestoreFlashCharValueFlag = 1;
    sleep_us(1000);                         // delay for flash operation
    cfg_led_event(LED_EVENT_FLASH_2HZ_5TIMES);
}

/************************************************************************
 *  					.
 ***********************************************************************/

_attribute_ram_code_ void  ble_remote_set_sleep_wakeup (u8 e, u8 *p, int n)
{
	if( blc_ll_getCurrentState() == BLS_LINK_STATE_CONN && ((u32)(bls_pm_getSystemWakeupTick() - clock_time())) > 80 * CLOCK_16M_SYS_TIMER_CLK_1MS){  //suspend time > 30ms.add gpio wakeup
		bls_pm_setWakeupSource(PM_WAKEUP_PAD);  //gpio pad wakeup suspend/deepsleep
	}
}



void 	app_switch_to_indirect_adv(u8 e, u8 *p, int n)
{

	bls_ll_setAdvParam( MY_ADV_INTERVAL_MIN, MY_ADV_INTERVAL_MAX,
						ADV_TYPE_CONNECTABLE_UNDIRECTED, app_own_address_type,
						0,  NULL,
						MY_APP_ADV_CHANNEL,
						ADV_FP_NONE);

	bls_ll_setAdvEnable(1);  //must: set adv enable
}



void 	ble_remote_terminate(u8 e,u8 *p, int n) //*p is terminate reason
{
	device_in_connection_state = 0;
	extern u8 PS_Transient_cryption_Enable;
	PS_Transient_cryption_Enable = 0;

	session_timeout_tick = 0;
	hap_transationID_valid_timeout = 0;
	hap_timed_write_timeout = 0;
	hap_procedure_valid_timeout = 0;
	hap_remove_pairing_timeout = 0;
	
	blt_adv_global_state_modify_once = 1;

	blt_set_scan_rsp_pkt(tbl_mac,DEVICE_SHORT_NAME,(u8 *)tbl_rsp);
	blt_set_normal_adv_pkt(HK_CATEGORY, DEVICE_SHORT_NAME);

	if(*p == HCI_ERR_CONN_TIMEOUT){

	}
	else if(*p == HCI_ERR_REMOTE_USER_TERM_CONN){  //0x13

	}
	else if(*p == HCI_ERR_CONN_TERM_MIC_FAILURE){

	}
	else{

	}

#if (BLE_APP_PM_ENABLE)
	 //user has push terminate pkt to ble TX buffer before deepsleep
	if(sendTerminate_before_enterDeep == 1){
		sendTerminate_before_enterDeep = 2;
	}
#endif

	advertise_begin_tick = clock_time();

	//if(ReCheckAndRestoreFlashCharValueFlag == 2)
	{
		remove_invalid_srpkey_timeout = clock_time() | 1;
	}

	ota_auth_enable = false;

	#if SOFTWARE_AUTHENTICATION_ENABLE
	CustVendorCmd_SoftAuth_wr_reset();
	CustVendorCmd_SoftAuth_rd_reset();
	#endif

	#if (BLT_SOFTWARE_TIMER_ENABLE)
	if(!pairPaired()){
		blt_soft_timer_tick_start();
		adv_blt_soft_timeout = blt_soft_timer_clock_time();
	}
	#endif

	USER_BLE_REMOTE_TERMINATE;
}




_attribute_ram_code_ void	user_set_rf_power (u8 e, u8 *p, int n)
{
	rf_set_power_level_index (MY_RF_POWER_INDEX);
}



void	task_connect (u8 e, u8 *p, int n)
{
	//bls_l2cap_requestConnParamUpdate (8, 8, 19, 200);  // 200mS
//	bls_l2cap_requestConnParamUpdate (8, 8, 99, 400);  // 1 S
//	bls_l2cap_requestConnParamUpdate (8, 8, 149, 600);  // 1.5 S
//	bls_l2cap_requestConnParamUpdate (8, 8, 199, 800);  // 2 S
//	bls_l2cap_requestConnParamUpdate (8, 8, 249, 800);  // 2.5 S
//	bls_l2cap_requestConnParamUpdate (8, 8, 299, 800);  // 3 S

	remove_invalid_srpkey_timeout = 0;

	hap_procedure_valid_timeout = clock_time () | 1;

	latest_user_event_tick = clock_time();

	device_in_connection_state = 1;//

	blt_adv_global_state_modify_once = 1;

	//+for apple home-kit.
	hk_l2cap_att_init ();

	//-for apple home-kit.
	interval_update_tick = clock_time() | 1; //none zero

	if (ReCheckADVStatusFlag == 0){
		ReCheckADVStatusFlag = 1;
	}
	if( broadcast_encryption_time_timeout || disconnect_event_time_timeout){
		broadcast_encryption_time_timeout = 0;	// clear broadcast encryption notification.
		disconnect_event_time_timeout = 0;
		// Todo: reset adv packet.
		blt_set_normal_adv_pkt(HK_CATEGORY, DEVICE_SHORT_NAME);
		bls_ll_setAdvInterval(MY_ADV_INTERVAL_MIN, MY_ADV_INTERVAL_MAX);
	}

	#if (BLT_SOFTWARE_TIMER_ENABLE)
	if(adv_blt_soft_timeout){
		adv_blt_soft_timeout = 0;
		blt_soft_timer_tick_stop();
	}
	#endif

	/*
	//Used only to test the correct passing of parameters to the user callback.
	printf("<%s:%d>\n", __FUNCTION__, __LINE__);
	printf("e = %X, n = %d\r\n", e, n);
	printf("p[%d] = {", n);
	for(int i = 0; i < n; i++)
		printf(" %X,", p[i]);
	printf("}\r\n");
	*/
	
	USER_TASK_CONNECT;
}


void	task_conn_update_req (u8 e, u8 *p, int n)
{

}

void	task_conn_update_done (u8 e, u8 *p, int n)
{

}

/*****************************************************************************************
 Note: battery check must do before any flash write/erase operation, cause flash write/erase
	   under a low or unstable power supply will lead to error flash operation

	   Some module initialization may involve flash write/erase, include: OTA initialization,
			SMP initialization, ..
			So these initialization must be done after  battery check
*****************************************************************************************/
_attribute_ram_code_ void user_init_battery_power_check(void)
{
	/*****************************************************************************************
	 Note: battery check must do before any flash write/erase operation, cause flash write/erase
		   under a low or unstable power supply will lead to error flash operation

		   Some module initialization may involve flash write/erase, include: OTA initialization,
				SMP initialization, ..
				So these initialization must be done after  battery check
	*****************************************************************************************/
	#if (BATT_CHECK_ENABLE)  //battery check must do before OTA relative operation
		do{
			if(analog_read(USED_DEEP_ANA_REG) & LOW_BATT_FLG){
				#if(VBAT_LEAKAGE_PROTECT_EN)
					app_battery_power_check(VBAT_DEEP_THRES_MV + 200, VBAT_SUSPEND_THRES_MV+100);  //2.2 V
				#else
					app_battery_power_check(VBAT_DEEP_THRES_MV + 200);
				#endif
			}
			else{
				#if(VBAT_LEAKAGE_PROTECT_EN)
					app_battery_power_check(VBAT_DEEP_THRES_MV, VBAT_SUSPEND_THRES_MV);
				#else
					app_battery_power_check(VBAT_DEEP_THRES_MV);
				#endif
			}
		}while(analog_read(USED_DEEP_ANA_REG) & LOW_BATT_SUSPEND_FLG);

	#endif
}

void user_init_normal(void)
{
	//random number generator must be initiated here( in the beginning of user_init_nromal)
	//when deepSleep retention wakeUp, no need initialize again
	random_generator_init();  //this is must

#if (BATT_CHECK_ENABLE)
	adc_power_on_sar_adc(1);
	battery_set_detect_enable(1);
#endif

	FUNC_NULL_KEEP_VAL();	//set name & OTA key of bin.

	dev_firmware_revision_get(); //get firmware_revision from version.h

// homekit init.
	COMMON_Init();

#if (FLASH_ADR_TYPE == FLASH_ADR_TYPE_DUAL_MODE)
	dev_flash_adr_table_init();
#endif

    dev_setup_code_set();
    dev_serial_number_set();
    dev_setup_id_set();

// lib func.
    dev_device_id_set();
    dev_hash_deviceid_set();

#if SOFTWARE_AUTHENTICATION_ENABLE
    dev_sw_auth_uuid_set(SW_AUTH_UUID_INIT, test_sw_uuid);
    dev_sw_auth_token_set(SW_AUTH_TOKEN_INIT, test_sw_token, sizeof(test_sw_token));
#endif

#if 0
    // test read broadcast encryption key.
    flash_read_long_val(flash_adr_layout.flash_adr_adv_encryption_key,0,bBroadcastEnKey,32);
#endif


    extern int factory_reset_handle();
    factory_reset_handle();

    pairInitID();

////////////////// BLE stack initialization ////////////////////////////////////
	u8  mac_random_static[6];
	blc_initMacAddress(CFG_ADR_MAC, tbl_mac, mac_random_static);
	
#if (FLASH_ADR_TYPE == FLASH_ADR_TYPE_DUAL_MODE)
	dev_random_static_mac_addr_set((unsigned long)flash_adr_layout.flash_adr_mac, false, tbl_mac, mac_random_static);
#endif

	#if(BLE_DEVICE_ADDRESS_TYPE == BLE_DEVICE_ADDRESS_PUBLIC)
		app_own_address_type = OWN_ADDRESS_PUBLIC;
	#elif(BLE_DEVICE_ADDRESS_TYPE == BLE_DEVICE_ADDRESS_RANDOM_STATIC)
		app_own_address_type = OWN_ADDRESS_RANDOM;
		blc_ll_setRandomAddr(mac_random_static);
	#endif

	blc_att_setRxMtuSize(HK_MTU_SIZE);

	////// Controller Initialization  //////////
	blc_ll_initBasicMCU();                      //mandatory
	blc_ll_initStandby_module(tbl_mac);				//mandatory
	blc_ll_initAdvertising_module(tbl_mac); 	//adv module: 		 mandatory for BLE slave,
	blc_ll_initSlaveRole_module();				//slave module: 	 mandatory for BLE slave,
	blc_ll_initPowerManagement_module();        //pm module:      	 optional




	///////////////////// USER application initialization ///////////////////
	//bls_ll_setAdvData( (u8 *)tbl_advData, sizeof(tbl_advData) );
	//bls_ll_setScanRspData( (u8 *)tbl_rsp, sizeof(tbl_rsp));			//homekit rsp ݺSDKݲһ
	blt_set_scan_rsp_pkt(tbl_mac,DEVICE_SHORT_NAME,(u8 *)tbl_rsp);

    ///////////////////// modify adv packet//////////////////////////////////////
	blt_set_normal_adv_pkt(HK_CATEGORY, DEVICE_SHORT_NAME);
    u16 hk_category = HK_CATEGORY;
    blt_adv_update_pkt(ADV_PKT_CATEGORY_FIELD, (const u8 *)&hk_category);

    u8 hk_setupHash[4];
    flash_read_page((unsigned long)(flash_adr_layout.flash_adr_hash_id), 4, hk_setupHash);
    blt_adv_update_pkt(ADV_PKT_SETUP_HASH, hk_setupHash);


	////// Host Initialization  //////////
	blc_gap_peripheral_init();    //gap initialization
	//extern void my_att_init ();
	my_att_init (); //gatt initialization
	
	//blc_l2cap_register_handler (blc_l2cap_packet_receive);  	//l2cap initialization
	blc_l2cap_register_handler (hk_l2cap_packet_receive);


	//Smp Initialization may involve flash write/erase(when one sector stores too much information,
	//   is about to exceed the sector threshold, this sector must be erased, and all useful information
	//   should re_stored) , so it must be done after battery check
#if (BLE_REMOTE_SECURITY_ENABLE)
	blc_smp_peripheral_init();
#else
	blc_smp_setSecurityLevel(No_Security);
#endif


	////////////////// config adv packet /////////////////////
#if (BLE_REMOTE_SECURITY_ENABLE)
	u8 bond_number = blc_smp_param_getCurrentBondingDeviceNumber();  //get bonded device number
	smp_param_save_t  bondInfo;
	if(bond_number)   //at least 1 bonding device exist
	{
		bls_smp_param_loadByIndex( bond_number - 1, &bondInfo);  //get the latest bonding device (index: bond_number-1 )

	}

	if(bond_number)   //set direct adv
	{
		//set direct adv
		u8 status = bls_ll_setAdvParam( MY_ADV_INTERVAL_MIN, MY_ADV_INTERVAL_MAX,
										ADV_TYPE_CONNECTABLE_DIRECTED_LOW_DUTY, app_own_address_type,
										bondInfo.peer_addr_type,  bondInfo.peer_addr,
										MY_APP_ADV_CHANNEL,
										ADV_FP_NONE);
		if(status != BLE_SUCCESS) { write_reg8(0x40002, 0x11); 	while(1); }  //debug: adv setting err

		//it is recommended that direct adv only last for several seconds, then switch to indirect adv
		bls_ll_setAdvDuration(MY_DIRECT_ADV_TMIE, 1);
		bls_app_registerEventCallback (BLT_EV_FLAG_ADV_DURATION_TIMEOUT, &app_switch_to_indirect_adv);

	}
	else   //set indirect adv
#endif
	{
		u8 status = bls_ll_setAdvParam(  MY_ADV_INTERVAL_MIN, MY_ADV_INTERVAL_MAX,
										 ADV_TYPE_CONNECTABLE_UNDIRECTED, app_own_address_type,
										 0,  NULL,
										 MY_APP_ADV_CHANNEL,
										 ADV_FP_NONE);
		if(status != BLE_SUCCESS) { write_reg8(0x40002, 0x11); 	while(1); }  //debug: adv setting err
	}

	bls_ll_setAdvEnable(0);  //adv disable.


	//set rf power index, user must set it after every suspend wakeup, cause relative setting will be reset in suspend
	user_set_rf_power(0, 0, 0);
	bls_app_registerEventCallback (BLT_EV_FLAG_SUSPEND_EXIT, &user_set_rf_power);



	//ble event call back
	bls_app_registerEventCallback (BLT_EV_FLAG_CONNECT, &task_connect);
	bls_app_registerEventCallback (BLT_EV_FLAG_TERMINATE, &ble_remote_terminate);


	bls_app_registerEventCallback (BLT_EV_FLAG_CONN_PARA_REQ, &task_conn_update_req);
	bls_app_registerEventCallback (BLT_EV_FLAG_CONN_PARA_UPDATE, &task_conn_update_done);

	//homekit && ble event call back
	extern void bls_set_advertise_prepare (void *p);
	bls_set_advertise_prepare(&task_adv_pre);


    hook_callback_func_register(Begin1stBootInitKeyPhase, (hook_callback_func_t)Begin1stBootInitKeyPhaseCB_pwm_hw_flash_on_cfg_1HZ);
    hook_callback_func_register(Finish1stBootInitKeyPhase, (hook_callback_func_t)Finish1stBootInitKeyPhaseCB_pwm_hw_flash_off);

    hook_callback_func_register(ReceivePairRequestPhase, (hook_callback_func_t)ReceivePairRequestPhaseCB);
    hook_callback_func_register(WaitSetupCodeInPhase, (hook_callback_func_t)WaitSetupCodeInPhaseCB);
    hook_callback_func_register(FinishSetupCodeInOrTripleTryPhase, (hook_callback_func_t)FinishSetupCodeInOrTripleTryPhaseCB);
    hook_callback_func_register(Register1stAdminKeyPhase, (hook_callback_func_t)Register1stAdminKeyPhaseCB);

    hook_callback_func_register(DoRemovePair1Phase, (hook_callback_func_t)DoRemovePair1PhaseCB);
    hook_callback_func_register(DoRemoveAllPairPhase, (hook_callback_func_t)DoRemoveAllPairPhaseCB);
	///////////////////// Power Management initialization///////////////////
#if(BLE_APP_PM_ENABLE)
	blc_ll_initPowerManagement_module();

	#if (PM_DEEPSLEEP_RETENTION_ENABLE)
		bls_pm_setSuspendMask (SUSPEND_ADV | DEEPSLEEP_RETENTION_ADV | SUSPEND_CONN | DEEPSLEEP_RETENTION_CONN);
		blc_pm_setDeepsleepRetentionThreshold(95, 95);
		blc_pm_setDeepsleepRetentionEarlyWakeupTiming( 220 );
		//blc_pm_setDeepsleepRetentionType(DEEPSLEEP_MODE_RET_SRAM_LOW32K); //default use 16k deep retention
	#else
		bls_pm_setSuspendMask (SUSPEND_ADV | SUSPEND_CONN);
	#endif

	bls_app_registerEventCallback (BLT_EV_FLAG_SUSPEND_ENTER, &ble_remote_set_sleep_wakeup);
#else
	bls_pm_setSuspendMask (SUSPEND_DISABLE);
#endif

	/*************************LED PWM INIT**************************/
	dev_char_val_update_from_flash();

	gpio_set_func(PWM_R, PMW_MODE_R);
	pwm_set_mode(PWMID_R, PWM_NORMAL_MODE);
	pwm_set_phase(PWMID_R, 0);	 //no phase at pwm beginning
	
	gpio_set_func(PWM_B, PMW_MODE_B);
	pwm_set_mode(PWMID_B, PWM_NORMAL_MODE);
	pwm_set_phase(PWMID_B, 0);	 //no phase at pwm beginning
	
	pwm_n_revert(PWMID_G);			// pwm_n, revert to be same to pwm.
	gpio_set_func(PWM_G, PMW_MODE_G);
	pwm_set_mode(PWMID_G, PWM_NORMAL_MODE);
	pwm_set_phase(PWMID_G, 0);	 //no phase at pwm beginning	
	
	pwm_n_revert(PWMID_W);			// pwm_n, revert to be same to pwm.
	REG_ADDR8(0x5AF) &= ~(BIT(0));
	REG_ADDR8(0x5AF) |= BIT(1); 	// must Set PD4 pin pad function as PWM2_N
	gpio_set_func(PWM_W, PMW_MODE_W);
	pwm_set_mode(PWMID_W, PWM_NORMAL_MODE);
	pwm_set_phase(PWMID_W, 0);	 //no phase at pwm beginning

#if LIGHTBULB_SERVICE_ENABLE
	extern void Convert_HSV2RGB(u32 * dbg);
	u32 dbg2 = 0;
	Convert_HSV2RGB(&dbg2);

	u8 led_val_tmp[6] = {0};
	
	if(light_onoff == 1)
	{
		memcpy(led_val_tmp, led_val, sizeof(led_val));
	}
	
	pwm_set_cycle_and_duty(PWMID_R, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16)led_val_tmp[0] );
	pwm_set_cycle_and_duty(PWMID_B, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16)led_val_tmp[1] );
	pwm_set_cycle_and_duty(PWMID_G, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16)led_val_tmp[2] );
	pwm_set_cycle_and_duty(PWMID_W, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16)led_val_tmp[3] );

	light_enable (light_onoff);
#else
	pwm_set_cycle_and_duty(PWMID_R, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16) (500 * CLOCK_SYS_CLOCK_1US) );
	pwm_set_cycle_and_duty(PWMID_B, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16) (500 * CLOCK_SYS_CLOCK_1US) );
	pwm_set_cycle_and_duty(PWMID_G, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16) (500 * CLOCK_SYS_CLOCK_1US) );
	pwm_set_cycle_and_duty(PWMID_W, (u16) (1000 * CLOCK_SYS_CLOCK_1US),  (u16) (500 * CLOCK_SYS_CLOCK_1US) );
#endif
	
	pwm_start(PWMID_R);
	pwm_start(PWMID_B);
	pwm_start(PWMID_G);
	pwm_start(PWMID_W);

	hk_led_ctrl_init();
	irq_enable();		// enable irq to proc time 1 interrupt.

	#if (MODULE_WATCHDOG_ENABLE)
	wd_stop();
	#endif

    // srp key initialization take 10 seonds on first power on
	#if HK_MULTI_SRP_PUBKEY_ENABLE
	srpInitKey_other(flash_adr_layout.flash_adr_srp_key, flash_adr_layout.flash_adr_srp_key_crc_data);
	srpInitKey_other(flash_adr_layout.flash_adr_srp_key1, flash_adr_layout.flash_adr_srp_key1_crc_data);
	srpInitKey_other(flash_adr_layout.flash_adr_srp_key2, flash_adr_layout.flash_adr_srp_key2_crc_data);
	srpInitKey_other(flash_adr_layout.flash_adr_srp_key3, flash_adr_layout.flash_adr_srp_key3_crc_data);
	extern int srpNum_pairSetup;
	srpNum_pairSetup = srpInitKey_valid_num_get();
	
	remove_invalid_srpkey_timeout = clock_time() | 1;
	#else
	srpInitKey();
	#endif

	#if (MODULE_WATCHDOG_ENABLE)
	wd_start();
	#endif
	
	usb_dp_pullup_en (1);  //open USB enum

	#if (BLT_SOFTWARE_TIMER_ENABLE)
	blt_soft_timer_init();
	if(!pairPaired()){
		blt_soft_timer_tick_start();
		adv_blt_soft_timeout = blt_soft_timer_clock_time();
	}
	#endif
	
    bls_ll_setAdvEnable(1);  //adv enable

	advertise_begin_tick = clock_time();

	//bls_ota_registerVendorCmdCb(&CustVendorCmdFunctions);			// ota vendor cmd, later proc.
	hkble_setCustVendorCmdCb(&CustVendorCmdFunctions);

	#if ENCODE_OTA_BIN_EN
	hkble_ota_registerFwDecryptCb(&hkble_ota_FwDecrypt);
	#endif

	#if OTA_FW_TYPE_VER_CHECK_EN
	hkble_ota_registerFwTypeVerCheckCb(&hkble_ota_FwTypeVerCheck);
	#endif

    broadcast_encryption_time_timeout = 0;
}




_attribute_ram_code_ void user_init_deepRetn(void)
{
#if (PM_DEEPSLEEP_RETENTION_ENABLE)

	blc_ll_initBasicMCU();   //mandatory
	rf_set_power_level_index (MY_RF_POWER_INDEX);

	blc_ll_recoverDeepRetention();

	DBG_CHN0_HIGH;    //debug

	irq_enable();
#endif
}


/////////////////////////////////////////////////////////////////////
// main loop flow
/////////////////////////////////////////////////////////////////////



void main_loop (void)
{
	#if (BLT_SOFTWARE_TIMER_ENABLE)
	blt_soft_timer_process(MAINLOOP_ENTRY);
	#endif

	////////////////////////////////////// BLE entry /////////////////////////////////
	blt_sdk_main_loop();

	if (blt_state == BLS_LINK_STATE_CONN )
	{
		hk_l2cap_att_main_loop();
	}

	#if (BATT_CHECK_ENABLE)
	if(battery_get_detect_enable() && clock_time_exceed(lowBattDet_tick, 500000) ){
		lowBattDet_tick = clock_time();
#if(VBAT_LEAKAGE_PROTECT_EN)
	
		if(analog_read(USED_DEEP_ANA_REG) & LOW_BATT_FLG){
			app_battery_power_check(VBAT_DEEP_THRES_MV+200, VBAT_SUSPEND_THRES_MV+100);
		}
		else
		{
			app_battery_power_check(VBAT_DEEP_THRES_MV, VBAT_SUSPEND_THRES_MV);
		}
#else
		app_battery_power_check(VBAT_DEEP_THRES_MV);  //2000 mV low battery
#endif
	}
	#endif

	////////////////////////////////////// UI entry /////////////////////////////////

	if(hkble_ota_data_check != HKBLE_OTA_DATA_NONE)
	{

#if (HKBLE_OTA_DATA_VERIFY_AFTER_SAVE_EN)
		extern int hkble_hk_ota_adr_index;
		extern void hkble_ota_verify_data_from_flash(void);
		extern void hkble_hk_ota_verify_data_from_flash(void);
		
		if(hkble_ota_data_check == HKBLE_OTA_DATA_END)
		{
			if( hkble_ota_adr_index >=0 )
			{
				hkble_ota_verify_data_from_flash();
				hkble_ota_data_check = HKBLE_OTA_DATA_NONE;
			}

			if( hkble_hk_ota_adr_index >= 0 )
			{
				hkble_hk_ota_verify_data_from_flash();
				hkble_ota_data_check = HKBLE_OTA_DATA_NONE;
			}
		}
		else if(hkble_ota_data_timeout && clock_time_exceed(hkble_ota_data_timeout, 4*1000*1000))
		{
			/*OTA data timeout*/
			if(hkble_ota_adr_index >= 0)
			{
				hkble_ota_verify_data_from_flash();
				hkble_ota_data_check = HKBLE_OTA_DATA_NONE;
			}

			if( hkble_hk_ota_adr_index >= 0 )
			{
				hkble_hk_ota_verify_data_from_flash();
				hkble_ota_data_check = HKBLE_OTA_DATA_NONE;
			}
		}
#else
		if(hkble_ota_data_timeout && clock_time_exceed(hkble_ota_data_timeout, 10000*1000))
		{
			/*OTA data timeout*/
			if((hkble_ota_adr_index >= 0) || (hkble_hk_ota_adr_index >= 0))
			{
				bls_ll_terminateConnection(0x13);
				rf_led_ota_error();
				start_reboot();
			}
		}
#endif
	}
	else
	{
#if LIGHTBULB_SERVICE_ENABLE

		if (ReCheckAndRestoreFlashCharValueFlag == 1)
		{
			ReCheckAndRestoreFlashCharValueFlag = 2;
			light_brightness = 100;
			hue_value = 0x420C0000;
			saturation_value = 0x429E0000;
			#if 1
			extern u8 dev_characteristic_store_to_flash(void);
			dev_characteristic_store_to_flash();
			#else
			flash_erase_sector(flash_char_value_addr);
			#endif
			extern void dev_char_val_restore_from_flash(void);
			dev_char_val_restore_from_flash();
		}

		// TODO:  bc_para_light_on	in ram, lost when power-on
		if( DoBroadcastEncrypDataCALL_flag && blt_state == BLS_LINK_STATE_ADV && (bc_para_light_on & BIT(7)))
		{
			broadcast_encryption_time_timeout = clock_time()|1;
			DoBroadcastEncrypDataCALL_flag = 0;
			u8 tmp_data[8];
			memset(tmp_data, 0, 8);
			tmp_data[0] = light_onoff;
			blt_set_encryption_broadcast_pkt(insts[LIGHTBULB_INST_NO+2], tmp_data);
			set_broadcast_adv_interval(bc_para_light_on);
		}

		#if CHAR_BRIGHTNESS_ENABLE
		if( DoBroadcastEncrypDataCALL_flag && blt_state == BLS_LINK_STATE_ADV && (bc_para_light_bri & BIT(7)))
		{
			broadcast_encryption_time_timeout = clock_time()|1;
			DoBroadcastEncrypDataCALL_flag = 0;
			int tmp_data[2];
			memset((u8*)tmp_data, 0, 8);
			tmp_data[0] = light_brightness;
			blt_set_encryption_broadcast_pkt(insts[LIGHTBULB_INST_NO+3], (u8 *)tmp_data);
			set_broadcast_adv_interval(bc_para_light_bri);
		}
		#endif

		#if CHAR_HUE_ENABLE
		if( DoBroadcastEncrypDataCALL_flag && blt_state == BLS_LINK_STATE_ADV && (bc_para_light_hue & BIT(7)))
		{
			broadcast_encryption_time_timeout = clock_time()|1;
			DoBroadcastEncrypDataCALL_flag = 0;
			u32 tmp_data[2];
			memset((u8*)tmp_data, 0, 8);
			tmp_data[0] = hue_value;
			blt_set_encryption_broadcast_pkt(insts[LIGHTBULB_INST_NO+4], (u8 *)tmp_data);
			set_broadcast_adv_interval(bc_para_light_hue);
		}
		#endif

		#if CHAR_SATURATION_ENABLE
		if( DoBroadcastEncrypDataCALL_flag && blt_state == BLS_LINK_STATE_ADV && (bc_para_light_satu & BIT(7)))
		{
			broadcast_encryption_time_timeout = clock_time()|1;
			DoBroadcastEncrypDataCALL_flag = 0;
			u32 tmp_data[2];
			memset((u8*)tmp_data, 0, 8);
			tmp_data[0] = saturation_value;
			blt_set_encryption_broadcast_pkt(insts[LIGHTBULB_INST_NO+5], (u8 *)tmp_data);
			set_broadcast_adv_interval(bc_para_light_satu);
		}
		#endif
#endif

		factory_reset_cnt_check();
		
		// save current characteristic value
		if(store_char_change_value_timeout && clock_time_exceed(store_char_change_value_timeout, 3000*1000)){
			store_char_change_value_timeout = 0;
			dev_characteristic_store_to_flash();
		}

		#if SOFTWARE_AUTHENTICATION_ENABLE
		CustVendorCmd_SoftAuth_timeout_check();
		#endif

		#if (BLT_SOFTWARE_TIMER_ENABLE)
		if(( blt_state == BLS_LINK_STATE_ADV) && adv_blt_soft_timeout && blt_soft_timer_clock_time_exceed(adv_blt_soft_timeout, 600-5) && (!pairPaired())){
			app_adv_state = 1;
			adv_blt_soft_timeout = 0;
			blt_soft_timer_tick_stop();
			bls_ll_setAdvEnable(0);  //adv disable.
		}
		#endif
		
		#if HK_SRP_PUBKEY_UPDATE_ENABLE
		if(remove_invalid_srpkey_timeout){

			if((clock_time_exceed(remove_invalid_srpkey_timeout, 30000*1000) && (!pairPaired())) || clock_time_exceed(remove_invalid_srpkey_timeout, 60000*1000)){
				
				remove_invalid_srpkey_timeout = 0;

				u32 time_tick = clock_time();

				extern int srpNum_pairSetup;
				extern bool srpkey2_exist;
				extern u8 srp_publickey_cnts;
				
				if(srpkey2_exist)
				{
					u8 srpNum_pairSetup_tmp;
					u8 srpNum;
					if(srpNum_pairSetup <= 0)
					{
						srpNum_pairSetup_tmp = 0;	//consume it start from 1st srp & remove the 2nd sector.
					}
					else
					{
						for(srpNum = 0; srpNum < srp_publickey_cnts; srpNum++)
						{
							if(srpNum_pairSetup & (1 << srpNum)) break;
						}
						srpNum_pairSetup_tmp = srpNum;
					}

					//if(srpInitKey_valid_num_get() <= 0)
					{
						if((srpNum_pairSetup_tmp == 1)|| (srpNum_pairSetup_tmp == 3))
							srpNum_pairSetup_tmp -= 1;
					}
					
					u32 flash_adr_srp_key = 0;
					u8 srpkey_valid_flag = 0xFF;
					if(srpNum_pairSetup_tmp == 2)
					{
						flash_adr_srp_key = (u32)(flash_adr_layout.flash_adr_srp_key);
						flash_read_page(flash_adr_srp_key + 2, 1, (u8 *)&srpkey_valid_flag);
					}
					else if(srpNum_pairSetup_tmp == 0)
					{
						flash_adr_srp_key = (u32)(flash_adr_layout.flash_adr_srp_key2);
						flash_read_page(flash_adr_srp_key + 2, 1, (u8 *)&srpkey_valid_flag);
					}

					if((srpNum_pairSetup_tmp == 0) || (srpNum_pairSetup_tmp == 2))
					{
						if(srpkey_valid_flag == SRPKEY_USED_IN_PAIR)	//0xFE: invalid flag
						{
							flash_erase_sector(flash_adr_srp_key);
							//REG_ADDR8(0x6F) = 0x20; 			   // rst all(act as power on reset)
							//while (1);
							//srpInitKey_other_without_led_ind(flash_adr_srp_key, flash_adr_srp_key_crc_data);
							
							#if (MODULE_WATCHDOG_ENABLE)
							wd_stop();
							#endif

							srpInitKey_other_without_led_ind(flash_adr_layout.flash_adr_srp_key, flash_adr_layout.flash_adr_srp_key_crc_data);
							srpInitKey_other_without_led_ind(flash_adr_layout.flash_adr_srp_key1, flash_adr_layout.flash_adr_srp_key1_crc_data);
							srpInitKey_other_without_led_ind(flash_adr_layout.flash_adr_srp_key2, flash_adr_layout.flash_adr_srp_key2_crc_data);
							srpInitKey_other_without_led_ind(flash_adr_layout.flash_adr_srp_key3, flash_adr_layout.flash_adr_srp_key3_crc_data);
							extern int srpNum_pairSetup;
							srpNum_pairSetup = srpInitKey_valid_num_get();

							#if (MODULE_WATCHDOG_ENABLE)
							wd_start();
							#endif

							remove_invalid_srpkey_timeout = clock_time() | 1;
							
							#if (BLT_SOFTWARE_TIMER_ENABLE)
							if(adv_blt_soft_timeout){
								/*it must adjust the soft timer clock becasue of the homekit srpkeys generation block the soft timer.*/
								u32 time_offset = ((u32)(remove_invalid_srpkey_timeout - time_tick) + CLOCK_SYS_CLOCK_1S - 1)/CLOCK_SYS_CLOCK_1S;
								time_tick = blt_soft_timer_clock_time() + time_offset;
								blt_soft_timer_clock_time_set(time_tick);
							}
							#endif
						}
					}
				}
			}
		}
		#endif
	}

}


