/********************************************************************************************************
 * @file     app_att.c
 *
 * @brief    for TLSR chips
 *
 * @author	 public@telink-semi.com;
 * @date     Sep. 18, 2018
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/
#define _APP_ATT_C_

#include "tl_common.h"

#include "stack/ble/ble.h"

#include "../../homekit_src/homekit_inc.h"
#include "../common/led_cfg.h"
#include "common.h"

#include "ble_UUID.h"
#include "hk_TopUUID.h"

#include "app_att.h"
#include "../../homekit_src/hk_ble_ll_ota.h"


extern void cfg_led_event(u32 e);
extern u16 ccc_GapAttServiceChanged;

/* ------------------------------------
    Macro Definitions
   ------------------------------------ */
#define U128_TELINK_SPP_UUID_SERVICE            {0x10,0x19,0x0d,0x0c,0x0b,0x0a,0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x02,0x01,0x00}       //!< TELINK_SPP service
#define U128_TELINK_SPP_DATA_SERVER2CLIENT      {0x11,0x19,0x0d,0x0c,0x0b,0x0a,0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x02,0x01,0x00}       //!< TELINK_SPP data from server to client
#define U128_TELINK_SPP_DATA_CLIENT2SERVER      {0x12,0x19,0x0d,0x0c,0x0b,0x0a,0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x02,0x01,0x00}       //!< TELINK_SPP data from client to server
#define U128_TELINK_SPP_DATA_OTA                {0x13,0x19,0x0d,0x0c,0x0b,0x0a,0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x02,0x01,0x00}       //!< TELINK_SPP data for ota
#define U128_TELINK_SPP_DATA_PAIR               {0x14,0x19,0x0d,0x0c,0x0b,0x0a,0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x02,0x01,0x00}       //!< TELINK_SPP data for ota

const u16 my_appearance         = GAP_APPEARE_UNKNOWN;

const u8 myServiceUUID[16]          = U128_TELINK_SPP_UUID_SERVICE;
const u8 TelinkSppDataPairUUID[16]  = U128_TELINK_SPP_DATA_PAIR;
const u8 my_OtaUUID[16]             = U128_TELINK_SPP_DATA_OTA;

const u8 my_OtaProp             = CHAR_PROP_READ | CHAR_PROP_WRITE_WITHOUT_RSP| CHAR_PROP_WRITE;
const u8 SppDataPairProp        = CHAR_PROP_READ | CHAR_PROP_WRITE;

const u16 my_FwRevisionStringUUID = CHARACTERISTIC_UUID_FW_REVISION_STRING;
const u8 my_FwRevisionStringProp = CHAR_PROP_READ;

u8 protocol_info_broadcast_param=1;
const hap_characteristic_desc_t hap_desc_char_protocol_info_A5_R_S = {hap_SecureRW_30, 0, CFG_DATA, 0, 0, 0, 0, 0, 0, 0,(u8*)&protocol_info_broadcast_param};
const hap_service_desc_t hap_desc_service_protocol_info_A5_R_S = {12, (0x00 + 0), 0, 0};

/* ------------------------------------
    Type Definitions
   ------------------------------------ */


/* ------------------------------------
    Variables Definitions
   ------------------------------------ */
//gap_periConnectParams_t my_periConnParameters = {20, 40, 0, 1000};

#define	VENDOR_ADD_SERVICE_EN               0
#if VENDOR_ADD_SERVICE_EN
#define VENDOR_ADD_SERVICE_attNum           4
const u8 DemoServiceUUID[16]            = {0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x20,0x19};
const u8 DemoDataClient2ServiceUUID[16] = {0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x21,0x19};
static const u8 DemoDataClient2ServerProp   = CHAR_PROP_READ | CHAR_PROP_WRITE_WITHOUT_RSP| CHAR_PROP_WRITE;
const u8 spp_demo_name[] = "demo";

extern int pair_dec_packet (u8 *ps);
extern u8 pair_login_ok;
_attribute_data_retention_ u8 demo_data_r[20] = {0xd0};

int demo_write_callback (void *ph)
{
	_attribute_data_retention_ static u8 demo_data_w[20] = {0};    // MAX 20
    rf_packet_att_data_t *p = (rf_packet_att_data_t *)ph;

    memset((void *)demo_data_w, 0, sizeof(demo_data_w));
    memcpy((void *)demo_data_w, (void *)(p->dat), p->l2cap - 3);
	memcpy((void *)demo_data_r, (void *)(p->dat), p->l2cap - 3);

    //blt_push_notify(0x004e, 0xabcd, 2); // 00 02 0a 06 00 04 00 12 4e 00 12 34 56   for ini

    return 1;
}

#else
#define VENDOR_ADD_SERVICE_attNum           0
#endif

#define ENABLE_CCC_SERVICE_CHANGED_RANGE_CHARACTERISTIC         0
#if ENABLE_CCC_SERVICE_CHANGED_RANGE_CHARACTERISTIC
u16 cccServiceChangedRange = 0;
#endif
_attribute_data_retention_ u8 identify = 0xff;
_attribute_data_retention_ u8 identifyCB_w = 0;

#ifdef MFI_ENABLE
const u8 my_pairFeature = 1;
#else
#if SOFTWARE_AUTHENTICATION_ENABLE
const u8 my_pairFeature = 2;	//for software authentication.
#else
const u8 my_pairFeature = 0;
#endif
#endif

_attribute_data_retention_ u8 otabuffer[20] = {0};
_attribute_data_retention_ u8 hap_desc_char_protocol_info_temp_var=0;

/* ------------------------------------
    Type Definitions
   ------------------------------------ */
const u16 my_gapAttUUID = SERVICE_UUID_GENERIC_ATTRIBUTE;
const u16 my_devInfoUUID = SERVICE_UUID_DEVICE_INFORMATION;

const u16 my_gapServiceUUID     = SERVICE_UUID_GENERIC_ACCESS;      // 0x1800

const u16 primaryServiceUUID    = GATT_UUID_PRIMARY_SERVICE;        // 0x2800   //!< Primary Service
const u16 characterUUID         = GATT_UUID_CHARACTER;              // 0x2803   //!< Characteristic

const u16 CEPUUID               = GATT_UUID_CHAR_EXT_PROPS;         // 0x2900   //!< Characteristic Extended Properties
const u16 userdesc_UUID         = GATT_UUID_CHAR_USER_DESC;         // 0x2901   //!< Characteristic User Description
const u16 ClientCharConfig_UUID = GATT_UUID_CLIENT_CHAR_CFG;        // 0x2902   //!< Client Characteristic Configuration
const u16 CFGUUID               = GATT_UUID_CHAR_PRESENT_FORMAT;    // 0x2904   //!< Characteristic Present Format
const u16 ValidRangeUUID        = GATT_UUID_VALID_RANGE;            // 0x2906   //!< Valid Range

const u16 my_devNameUUID        = GATT_UUID_DEVICE_NAME;            // 0x2a00   //!< Report Reference
const u16 my_appearanceUUID     = GATT_UUID_APPEARANCE;             // 0x2a01   //!< Report Reference
const u16 my_periConnParamUUID  = GATT_UUID_PERI_CONN_PARAM;    	// 0x2a04
const u16 my_ServiceChangedUUID = GATT_UUID_SERVICE_CHANGE;        	// 0x2a05   //!< Report Reference

//-const u16 CCCnoneProp           = 0x000;                            // 0x0000
const u16 CCCnotifyOnlyProp     = CLIENT_CHAR_CFG_NOTI;             // 0x0001 //!< permit broadcasts of the Characteristic Value
const u16 CCCindicationOnlyProp = CLIENT_CHAR_CFG_IND;              // 0x0002 //!< permit reads of the Characteristic Value

const u8 writeOnlyProp          = CHAR_PROP_WRITE;
const u8 writeNotifyProp        = CHAR_PROP_WRITE | CHAR_PROP_NOTIFY;
const u8 readOnlyProp           = CHAR_PROP_READ;
const u8 readIndicateProp       = CHAR_PROP_READ | CHAR_PROP_INDICATE;
const u8 readWriteProp          = CHAR_PROP_WRITE | CHAR_PROP_READ;
const u8 notifyOnlyProp         = CHAR_PROP_NOTIFY;
const u8 readWriteIndicateProp  = CHAR_PROP_WRITE | CHAR_PROP_READ | CHAR_PROP_INDICATE;
const u8 readWriteNotifyProp    = CHAR_PROP_WRITE | CHAR_PROP_READ | CHAR_PROP_NOTIFY;
const u8 ServiceChangedProp     = CHAR_PROP_READ | CHAR_PROP_INDICATE;  // 0x22;

/////////////////////////////////////////////////////////////
//  attribute_t.attrMaxLen = 0: pair read/write:
/////////////////////////////////////////////////////////////
const attribute_t my_Attributes[] = {
#if ENABLE_CCC_SERVICE_CHANGED_RANGE_CHARACTERISTIC
    {5+HOMEKIT_NO+4+6+VENDOR_ADD_SERVICE_attNum,0,0,0,0,0,0,0},
#else
    {5+HOMEKIT_NO+4-4+6+VENDOR_ADD_SERVICE_attNum,0,0,0,0,0,0,0},
#endif

    // GATT
    {5,ATT_PERMISSIONS_READ, 2,2,2, (u8*)(&primaryServiceUUID), (u8*)(&my_gapServiceUUID), 0, 0},
    	{0,ATT_PERMISSIONS_READ,2,1,1, (u8*)(&characterUUID), (u8*)(&readOnlyProp), 0, 0},
    	{0,ATT_PERMISSIONS_READ,2, sizeof (my_devName), sizeof (my_devName),(u8*)(&my_devNameUUID), (u8*)(my_devName), 0, 0},
    	{0,ATT_PERMISSIONS_READ,2,1,1, (u8*)(&characterUUID), (u8*)(&readOnlyProp), 0, 0},
    	{0,ATT_PERMISSIONS_READ,2, sizeof (my_appearance), sizeof (my_appearance),(u8*)(&my_appearanceUUID), (u8*)(&my_appearance), 0, 0},
    	//////////////////////////////////////////////////
    	//{0,2,1,1, (u8*)(&characterUUID), (u8*)(&readOnlyProp), 0, 0},
    	//{0,2, sizeof (my_periConnParameters), sizeof (my_periConnParameters),(u8*)(&my_periConnParamUUID), (u8*)(&my_periConnParameters), 0, 0},
    	//////////////////////////////////////////////////


    //////////////////////////////////////////////////
    HOMEKIT_TOP_ATT_TABLE

    ////////////////// Generic Attribute ////////////////////////////////////
#if ENABLE_CCC_SERVICE_CHANGED_RANGE_CHARACTERISTIC
    {4,ATT_PERMISSIONS_READ,2,2,2, (u8*)(&primaryServiceUUID), (u8*)(&my_gapAttUUID), 0, 0},
        {0,ATT_PERMISSIONS_READ,2,1,1, (u8*)(&characterUUID), (u8*)(&ServiceChangedProp), 0, 0},
        {0,ATT_PERMISSIONS_READ,2, sizeof (ServiceChangedRange), sizeof (ServiceChangedRange),(u8*)(&my_ServiceChangedUUID), (u8*)(ServiceChangedRange), 0, 0},
        {0,ATT_PERMISSIONS_RDWR,2,2,2, (u8*)(&ClientCharConfig_UUID), (u8*)(&ccc_GapAttServiceChanged), ccc_GapAttServiceChangedWriteCB, 0},
#endif

        ////////////////// Device Info ////////////////////////////////////
    {6,ATT_PERMISSIONS_READ,2,2,2, (u8*)(&primaryServiceUUID), (u8*)(&my_devInfoUUID), 0, 0},
        {0,ATT_PERMISSIONS_READ,2,1,1, (u8*)(&characterUUID), (u8*)(&my_OtaProp), 0, 0},
        //{0,ATT_PERMISSIONS_RDWR,16,20,20, (u8*)my_OtaUUID, otabuffer, &otaWrite, &otaRead},	//general ble OTA.
        {0,ATT_PERMISSIONS_RDWR,16,20,20, (u8*)my_OtaUUID, otabuffer, &hkble_otaWrite, &hkble_otaRead},	//homekit ble OTA.
        {0,ATT_PERMISSIONS_READ,2, sizeof (my_OtaName), sizeof (my_OtaName),(u8*)(&userdesc_UUID), (u8*)(my_OtaName), 0, 0},

		{0,ATT_PERMISSIONS_READ,2,1,1,(u8*)(&characterUUID), (u8*)(&my_FwRevisionStringProp), 0, 0},
		{0,ATT_PERMISSIONS_READ,2,sizeof (firmware_revision),sizeof (firmware_revision),(u8*)(&my_FwRevisionStringUUID), (u8*)(firmware_revision), 0, 0},

        //{0,ATT_PERMISSIONS_READ,2,1,1,(u8*)(&characterUUID), (u8*)(&SppDataPairProp), 0, 0},
//--        {0,16,1,1,(u8*)(TelinkSppDataPairUUID), 0, &pairWrite, &pairRead},
        //{0,ATT_PERMISSIONS_READ,16,1,1,(u8*)(TelinkSppDataPairUUID), 0, 0, 0},
        //{0,ATT_PERMISSIONS_READ,2,sizeof (spp_pairname), sizeof (spp_pairname),(u8*)(&userdesc_UUID), (u8*)(spp_pairname), 0, 0},

#if VENDOR_ADD_SERVICE_EN
    //add service ------------- demo code
    {VENDOR_ADD_SERVICE_attNum,ATT_PERMISSIONS_READ,2,16,16, (u8*)(&primaryServiceUUID), (u8*)(DemoServiceUUID), 0, 0},
        {0,ATT_PERMISSIONS_READ,2,1,1, (u8*)(&characterUUID), (u8*)(&DemoDataClient2ServerProp), 0, 0},
        {0,ATT_PERMISSIONS_RDWR,16,20,20, (u8*)(DemoDataClient2ServiceUUID), (demo_data_r), &demo_write_callback, 0},
        {0,ATT_PERMISSIONS_READ,2,sizeof (spp_demo_name), sizeof (spp_demo_name),(u8*)(&userdesc_UUID), (u8*)(spp_demo_name), 0, 0},
#endif

};

/* ------------------------------------
    Function Prototypes
   ------------------------------------ */
int identifyWriteCallback(void *pp)
{
    identifyCB_w++;
    _attribute_data_retention_ static u8 identifyCB_w_previous=0;

    extern rf_packet_adv_t	pkt_adv;
    extern u8 blt_pair_ok;

    if(pkt_adv.data[ADV_PKT_PAIR_STATUS_FLAG])
    {
        if(identifyCB_w != identifyCB_w_previous)
        {
            identifyCB_w_previous = identifyCB_w;
            cfg_led_event(LED_EVENT_FLASH_1HZ_3TIMES);
        }
    }
    else
    {
        if(blt_pair_ok)
        {
            if(identifyCB_w != identifyCB_w_previous)
            {
                identifyCB_w_previous = identifyCB_w;
                cfg_led_event(LED_EVENT_FLASH_1HZ_3TIMES);
                extern u32 session_timeout_tick;
                session_timeout_tick = clock_time () | 1;
            }
        }
		else
		{
			return -1;	//report HAP_STCODE_INSF_AUTHE to controller.
		}
    }

    return 0;
}

#if (MODULE_WATCHDOG_ENABLE)
int pairSetupWriteCallback(void *pp)
{
	wd_stop();
	int ret = pairSetupWriteInnerCallback(pp);
	wd_start();
	return ret;
}

int pairVerifyWriteCallback(void *pp)
{
	wd_stop();
	int ret = pairVerifyWriteInnerCallback(pp);
	wd_start();
	return ret;
}
#else
int pairSetupWriteCallback(void *pp)
{
	return pairSetupWriteInnerCallback(pp);
}

int pairVerifyWriteCallback(void *pp)
{
	return pairVerifyWriteInnerCallback(pp);
}
#endif


void	my_att_init (void)
{
	bls_att_setAttributeTable ((u8 *)my_Attributes);
}


