/********************************************************************************************************
 * @file     bin_name_key_setting.h
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2010, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/

#pragma once

#include "../../tl_common.h"

/*
 * 1. include this file only in the C file with init function, such as user_init(), 
 * 2. call FUNC_NULL_KEEP_VAL() in this function.
 * 3. update tl_auth_check_fw.exe and tl_check_fw.sh. slect AGTHM 2.
*/


#if (!WIN32)    // also used for shell file.
__WEAK void function_null_compile(const void *p){}// just for avoid being optimized

#if ENCODE_OTA_BIN_EN
/**
 * key_encode_bin : Encryption key for OTA firmware.
 * eclipse will create two firmwares: *.bin and *_enc.bin . "*_enc.bin" is only used for OTA.
 * The key is used to encrypte in eclipse and decrypte in firmware.
 * Key size must be 16 bytes in hex and should never be changed any more for this product.
*/
#ifndef ENCODE_BIN_USER_KEY     // please define in "user_app_config.h"
#define ENCODE_BIN_USER_KEY     {0} //{0x51,0x03,0x1f,0x03,0x57,0x81,0x7b,0x5c,0x48,0x83,0x93,0xae,0xa8,0xc6,0x5d,0x9a,} // 
#endif
const u8 key_encode_bin[] = ENCODE_BIN_USER_KEY;    // must const
STATIC_ASSERT(sizeof(key_encode_bin) == 16);

#ifdef ENCODE_BIN_USER_NAME     // please define in "user_app_config.h" if need.
const u8 ENCODE_BIN_USER_NAME_CONST[] = ENCODE_BIN_USER_NAME; // {"8258_mesh_AES"}
STATIC_ASSERT(sizeof(ENCODE_BIN_USER_NAME_CONST) >= 1);
#define FUNC_NULL_KEEP_ENCODE_BIN_USER_NAME()   do{function_null_compile(ENCODE_BIN_USER_NAME_CONST);}while(0)
#endif
#endif

#ifndef FUNC_NULL_KEEP_ENCODE_BIN_USER_NAME
#define FUNC_NULL_KEEP_ENCODE_BIN_USER_NAME()    
#endif

#ifdef NORMAL_BIN_USER_NAME    // please define in "user_app_config.h" if need.
const u8 NORMAL_BIN_USER_NAME_CONST[] = NORMAL_BIN_USER_NAME; // {"8258_mesh_normal"}
STATIC_ASSERT(sizeof(NORMAL_BIN_USER_NAME_CONST) >= 1);
#define FUNC_NULL_KEEP_NORMAL_BIN_USER_NAME()     do{function_null_compile(NORMAL_BIN_USER_NAME_CONST);}while(0)
#else
#define FUNC_NULL_KEEP_NORMAL_BIN_USER_NAME()     
#endif

#define FUNC_NULL_KEEP_VAL()    FUNC_NULL_KEEP_ENCODE_BIN_USER_NAME(); \
                                FUNC_NULL_KEEP_NORMAL_BIN_USER_NAME();
#else
#define FUNC_NULL_KEEP_VAL()    
#endif



