/********************************************************************************************************
 * @file     hk_Valve2Service.c
 *
 * @brief    for TLSR chips
 *
 * @author	 telink
 * @date     Sep. 30, 2010
 *
 * @par      Copyright (c) 2016, Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *           
 *			 The information contained herein is confidential and proprietary property of Telink 
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms 
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai) 
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in. 
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this 
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided. 
 *           
 *******************************************************************************************************/
#define _HK_VALVE2_SERVICE_C_
 
 
/* ------------------------------------
    Header Files
   ------------------------------------ */
#include "tl_common.h"
#include "stack/ble/ble.h"
#include "hk_Valve2Service.h"
#include "ble_UUID.h"
#include "hk_TopUUID.h"

extern u32 session_timeout_tick;
extern u8 EventNotifyCNT;
extern store_char_change_value_t* char_save_restore;
extern st_ll_conn_slave_t		bltc;

extern void assign_current_flash_char_value_addr(void);
extern void store_char_value_to_flash(void);

/* ------------------------------------
    Macro Definitions
   ------------------------------------ */


/* ------------------------------------
    Type Definitions
   ------------------------------------ */


/* ------------------------------------
    Variables Definitions
   ------------------------------------ */
unsigned char hValve2Active = 0;
unsigned char hValve2InUse = 0;
unsigned char hValve2ValveType = 1;	//for Irrigation System
unsigned int hValve2SetDuration = 0;
unsigned int hValve2PreviousSetDuration = 0;
unsigned int hValve2RemainingDuration = 0;
unsigned char hValve2IsConfigured = 1; //for the valve used in irrigation system
unsigned char hValve2ServiceLabelIndex = 2;	//for the 2nd index
unsigned char hValve2StatusFault = 0;

unsigned char VAR_VALVE2_UNKOWN=0;

unsigned short ccc_hValve2Active = 0;
unsigned short ccc_hValve2InUse = 0;
unsigned short ccc_hValve2ValveType = 0;
unsigned short ccc_hValve2SetDuration = 0;
unsigned short ccc_hValve2RemainingDuration = 0;
unsigned short ccc_hValve2IsConfigured = 0;
unsigned short ccc_hValve2StatusFault = 0;


#if VALVE2_ACTIVE_BRC_ENABLE
    unsigned char bc_para_hValve2Active = 1;
#endif

#if VALVE2_INUSE_BRC_ENABLE
    unsigned char bc_para_hValve2InUse = 1;
#endif

#if VALVE2_VALVETYPE_BRC_ENABLE
    unsigned char bc_para_hValve2ValveType = 1;
#endif

#if VALVE2_SETDURATION_BRC_ENABLE
    unsigned char bc_para_hValve2SetDuration = 1;
#endif

#if VALVE2_REMAININGDURATION_BRC_ENABLE
    unsigned char bc_para_hValve2RemainingDuration = 1;
#endif

#if VALVE2_ISCONFIGURED_BRC_ENABLE
    unsigned char bc_para_hValve2IsConfigured = 1;
#endif

#if VALVE2_STATUSFAULT_BRC_ENABLE
    unsigned char bc_para_hValve2StatusFault = 1;
#endif


const u8 hValve2ServiceName[13] = "Valve2Service";
//const u8 hValve2ServiceName[13] = "";	//for TCV003

const u8 VALVE2_ACTIVE_VALID_RANGE[2] = {0x00, 0x01};
const u8 VALVE2_INUSE_VALID_RANGE[2] = {0x00, 0x01};
const u8 VALVE2_VALVETYPE_VALID_RANGE[2] = {0x00, 0x03};
const u8 VALVE2_SETDURATION_VALID_RANGE[8] = {0x00, 0x00, 0x00, 0x00, 0x10, 0x0E, 0x00, 0x00};  /* 0~3600 */
const u8 VALVE2_REMAININGDURATION_VALID_RANGE[8] = {0x00, 0x00, 0x00, 0x00, 0x10, 0x0E, 0x00, 0x00};  /* 0~3600 */
const u8 VALVE2_ISCONFIGURED_VALID_RANGE[2] = {0x00, 0x01};
const u8 VALVE2_SERVICELABELINDEX_VALID_RANGE[2] = { 0x01, 0xFF};
const u8 VALVE2_STATUSFAULT_VALID_RANGE[2] = {0x00, 0x01};

const u8 VALVE2_ACTIVE_VALID_STEP = 0x01;
const u8 VALVE2_INUSE_VALID_STEP = 0x01;
const u8 VALVE2_VALVETYPE_VALID_STEP = 0x01;
const u8 VALVE2_SETDURATION_VALID_STEP[4] = {0x01, 0x00, 0x00, 0x00};
const u8 VALVE2_REMAININGDURATION_VALID_STEP[4] = {0x01, 0x00, 0x00, 0x00};
const u8 VALVE2_ISCONFIGURED_VALID_STEP = 0x01;
const u8 VALVE2_SERVICELABELINDEX_VALID_STEP = 0x01;
const u8 VALVE2_STATUSFAULT_VALID_STEP = 0x01;

const hap_characteristic_desc_t hap_desc_char_VALVE2_ACTIVE_R_S = {hap_PReadWrite_NOTIFY, 0, CFG_UINT8, 0, 2, 1, 0, 0, VALVE2_ACTIVE_VALID_RANGE, &VALVE2_ACTIVE_VALID_STEP, BC_PARA_VALVE2_ACTIVE};
const hap_characteristic_desc_t hap_desc_char_VALVE2_INUSE_R_S = {hap_PRead_NOTIFY, 0, CFG_UINT8, 0, 2, 1, 0, 0, VALVE2_INUSE_VALID_RANGE, &VALVE2_INUSE_VALID_STEP, BC_PARA_VALVE2_INUSE};
const hap_characteristic_desc_t hap_desc_char_VALVE2_VALVETYPE_R_S = {hap_PRead_NOTIFY, 0, CFG_UINT8, 0, 2, 1, 0, 0, VALVE2_VALVETYPE_VALID_RANGE, &VALVE2_VALVETYPE_VALID_STEP, BC_PARA_VALVE2_VALVETYPE};
const hap_characteristic_desc_t hap_desc_char_VALVE2_SETDURATION_R_S = {hap_PReadWrite_NOTIFY, 0, CFG_UINT32, 0, 8, 4, 0, 0, VALVE2_SETDURATION_VALID_RANGE, VALVE2_SETDURATION_VALID_STEP, BC_PARA_VALVE2_SETDURATION};
const hap_characteristic_desc_t hap_desc_char_VALVE2_REMAININGDURATION_R_S = {hap_PRead_NOTIFY, 0, CFG_UINT32, 0, 8, 4, 0, 0, VALVE2_REMAININGDURATION_VALID_RANGE, VALVE2_REMAININGDURATION_VALID_STEP, BC_PARA_VALVE2_REMAININGDURATION};
#if VALVE2_ISCONFIGURED_SUPPORT_PAIRED_WRITE	//Is Configured char has optional permission for paired write in HAP R14.
const hap_characteristic_desc_t hap_desc_char_VALVE2_ISCONFIGURED_R_S = {hap_PReadWrite_NOTIFY, 0, CFG_UINT8, 0, 2, 1, 0, 0, VALVE2_ISCONFIGURED_VALID_RANGE, &VALVE2_ISCONFIGURED_VALID_STEP, BC_PARA_VALVE2_ISCONFIGURED};
#else
const hap_characteristic_desc_t hap_desc_char_VALVE2_ISCONFIGURED_R_S = {hap_PRead_NOTIFY, 0, CFG_UINT8, 0, 2, 1, 0, 0, VALVE2_ISCONFIGURED_VALID_RANGE, &VALVE2_ISCONFIGURED_VALID_STEP, BC_PARA_VALVE2_ISCONFIGURED};
#endif
const hap_characteristic_desc_t hap_desc_char_VALVE2_SERVICELABELINDEX_R_S = {hap_SecureR_10, 0, CFG_UINT8, 0, 2, 1, 0, 0, VALVE2_SERVICELABELINDEX_VALID_RANGE, &VALVE2_SERVICELABELINDEX_VALID_STEP, 0};
const hap_characteristic_desc_t hap_desc_char_VALVE2_STATUSFAULT_R_S = {hap_PRead_NOTIFY, 0, CFG_UINT8, 0, 2, 1, 0, 0, VALVE2_STATUSFAULT_VALID_RANGE, &VALVE2_STATUSFAULT_VALID_STEP, BC_PARA_VALVE2_STATUSFAULT};

const hap_characteristic_desc_t hap_desc_char_valve2_A5_R_S = {hap_SecureRW_30, 0, CFG_DATA, 0, 0, 0, 0, 0, 0, 0, 0};

#if VALVE2_LINKED_SERVICES_ENABLE
/*fill the array with the pointer of linked services instance id.*/
const u16 * hValve2_linked_svcs[] = {};
#endif

#if(VALVE2_IS_HIDDEN_SERVICE == 1)
const hap_service_desc_t hap_desc_service_valve2_A5_R_S = {VALVE2_INST_NO+1, (0x02 + VALVE2_AS_PRIMARY_SERVICE), VALVE2_LINKED_SVCS_LENGTH, VALVE2_LINKED_SVCS};
#else
const hap_service_desc_t hap_desc_service_valve2_A5_R_S = {VALVE2_INST_NO+1, (0x00 + VALVE2_AS_PRIMARY_SERVICE), VALVE2_LINKED_SVCS_LENGTH, VALVE2_LINKED_SVCS};
#endif


/* ------------------------------------
    Function Prototypes
   ------------------------------------ */
int Valve2ActiveReadCallback(void *pp)
{
	session_timeout_tick = clock_time () | 1;
	return 1;
}

int Valve2ActiveWriteCallback(void *pp)
{
	blt_adv_set_global_state ();

	if (hap_char_boundary_check()) {
		assign_current_flash_char_value_addr();
	#if VALVE2_SERVICE_ENABLE
		VALVE2_STORE_CHAR_VALUE_T_ACTIVE_RESTORE
	#endif
	} else {
		store_char_value_to_flash();
	}

	session_timeout_tick = clock_time () | 1;
	return 1;
}

int Valve2InUseReadCallback(void *pp)
{
    session_timeout_tick = clock_time () | 1;
    return 1;
}

int Valve2ValveTypeReadCallback(void *pp)
{
    session_timeout_tick = clock_time () | 1;
    return 1;
}

int Valve2SetDurationReadCallback(void *pp)
{
    session_timeout_tick = clock_time () | 1;
    return 1;
}

int Valve2SetDurationWriteCallback(void *pp)
{
	blt_adv_set_global_state ();

	if (hap_char_boundary_check()) {
		assign_current_flash_char_value_addr();
	#if VALVE2_SERVICE_ENABLE
		VALVE2_STORE_CHAR_VALUE_T_SETDURATION_RESTORE
	#endif
	} else {
		store_char_value_to_flash();
		
		if(hValve2InUse == 1)
		{
			if(hValve2SetDuration > hValve2PreviousSetDuration)
				hValve2RemainingDuration = hValve2SetDuration - hValve2PreviousSetDuration;
			else
				hValve2RemainingDuration = hValve2SetDuration;
			
			if(EventNotifyCNT)
				if(bltc.conn_terminate_pending == 0)
				{
					if(indicate_handle_no_send_once_after_checkit(VALVE2_HANDLE_NUM_REMAININGDURATION) == 1)
						sleep_us(250);
				}
		}
		
		hValve2PreviousSetDuration = hValve2SetDuration;
	}

	session_timeout_tick = clock_time () | 1;
	return 1;
}

int Valve2RemainingDurationReadCallback(void *pp)
{
    session_timeout_tick = clock_time () | 1;
    return 1;
}

int Valve2IsConfiguredReadCallback(void *pp)
{
    session_timeout_tick = clock_time () | 1;
    return 1;
}

int Valve2IsConfiguredWriteCallback(void *pp)
{
	blt_adv_set_global_state ();

	if (hap_char_boundary_check()) {
		assign_current_flash_char_value_addr();
	#if VALVE2_SERVICE_ENABLE
		VALVE2_STORE_CHAR_VALUE_T_ISCONFIGURED_RESTORE
	#endif
	} else {
		store_char_value_to_flash();
	}

	session_timeout_tick = clock_time () | 1;
	return 1;
}

int Valve2ServiceLabelIndexReadCallback(void *pp)
{
	session_timeout_tick = clock_time () | 1;
	return 1;
}


int Valve2StatusFaultReadCallback(void *pp)
{
    session_timeout_tick = clock_time () | 1;
    return 1;
}


int ccc_Valve2ActiveWriteCB(void *pp)
{
	static u16 ccc_hValve2Active_pre = 0;

	if(ccc_hValve2Active_pre != ccc_hValve2Active)
	{
		if(ccc_hValve2Active == 0)
		{
			ccc_hValve2Active_pre = ccc_hValve2Active;
			EventNotifyCNT--;
			indicate_handle_no_remove(VALVE2_HANDLE_NUM_ACTIVE);
		}
		else
		{
			ccc_hValve2Active_pre = ccc_hValve2Active;
			EventNotifyCNT++;
			indicate_handle_no_reg(VALVE2_HANDLE_NUM_ACTIVE);
		}
	}
	session_timeout_tick = clock_time () | 1;
	return 1;
}

int ccc_Valve2InUseWriteCB(void *pp)
{
	static u16 ccc_hValve2InUse_pre = 0;

	if(ccc_hValve2InUse_pre != ccc_hValve2InUse)
	{
		if(ccc_hValve2InUse == 0)
		{
			ccc_hValve2InUse_pre = ccc_hValve2InUse;
			EventNotifyCNT--;
			indicate_handle_no_remove(VALVE2_HANDLE_NUM_INUSE);
		}
		else
		{
			ccc_hValve2InUse_pre = ccc_hValve2InUse;
			EventNotifyCNT++;
			indicate_handle_no_reg(VALVE2_HANDLE_NUM_INUSE);
		}
	}
	session_timeout_tick = clock_time () | 1;
	return 1;
}

int ccc_Valve2ValveTypeWriteCB(void *pp)
{
	static u16 ccc_hValve2ValveType_pre = 0;

	if(ccc_hValve2ValveType_pre != ccc_hValve2ValveType)
	{
		if(ccc_hValve2ValveType == 0)
		{
			ccc_hValve2ValveType_pre = ccc_hValve2ValveType;
			EventNotifyCNT--;
			indicate_handle_no_remove(VALVE2_HANDLE_NUM_VALVETYPE);
		}
		else
		{
			ccc_hValve2ValveType_pre = ccc_hValve2ValveType;
			EventNotifyCNT++;
			indicate_handle_no_reg(VALVE2_HANDLE_NUM_VALVETYPE);
		}
	}
	session_timeout_tick = clock_time () | 1;
	return 1;
}

int ccc_Valve2SetDurationWriteCB(void *pp)
{
	static u16 ccc_hValve2SetDuration_pre = 0;

	if(ccc_hValve2SetDuration_pre != ccc_hValve2SetDuration)
	{
		if(ccc_hValve2SetDuration == 0)
		{
			ccc_hValve2SetDuration_pre = ccc_hValve2SetDuration;
			EventNotifyCNT--;
			indicate_handle_no_remove(VALVE2_HANDLE_NUM_SETDURATION);
		}
		else
		{
			ccc_hValve2SetDuration_pre = ccc_hValve2SetDuration;
			EventNotifyCNT++;
			indicate_handle_no_reg(VALVE2_HANDLE_NUM_SETDURATION);
		}
	}
	session_timeout_tick = clock_time () | 1;
	return 1;
}

int ccc_Valve2RemainingDurationWriteCB(void *pp)
{
	static u16 ccc_hValve2RemainingDuration_pre = 0;

	if(ccc_hValve2RemainingDuration_pre != ccc_hValve2RemainingDuration)
	{
		if(ccc_hValve2RemainingDuration == 0)
		{
			ccc_hValve2RemainingDuration_pre = ccc_hValve2RemainingDuration;
			EventNotifyCNT--;
			indicate_handle_no_remove(VALVE2_HANDLE_NUM_REMAININGDURATION);
		}
		else
		{
			ccc_hValve2RemainingDuration_pre = ccc_hValve2RemainingDuration;
			EventNotifyCNT++;
			indicate_handle_no_reg(VALVE2_HANDLE_NUM_REMAININGDURATION);
		}
	}
	session_timeout_tick = clock_time () | 1;
	return 1;
}

int ccc_Valve2IsConfiguredWriteCB(void *pp)
{
	static u16 ccc_hValve2IsConfigured_pre = 0;

	if(ccc_hValve2IsConfigured_pre != ccc_hValve2IsConfigured)
	{
		if(ccc_hValve2IsConfigured == 0)
		{
			ccc_hValve2IsConfigured_pre = ccc_hValve2IsConfigured;
			EventNotifyCNT--;
			indicate_handle_no_remove(VALVE2_HANDLE_NUM_ISCONFIGURED);
		}
		else
		{
			ccc_hValve2IsConfigured_pre = ccc_hValve2IsConfigured;
			EventNotifyCNT++;
			indicate_handle_no_reg(VALVE2_HANDLE_NUM_ISCONFIGURED);
		}
	}
	session_timeout_tick = clock_time () | 1;
	return 1;
}

int ccc_Valve2StatusFaultWriteCB(void *pp)
{
	static u16 ccc_hValve2StatusFault_pre = 0;

	if(ccc_hValve2StatusFault_pre != ccc_hValve2StatusFault)
	{
		if(ccc_hValve2StatusFault == 0)
		{
			ccc_hValve2StatusFault_pre = ccc_hValve2StatusFault;
			EventNotifyCNT--;
			indicate_handle_no_remove(VALVE2_HANDLE_NUM_STATUSFAULT);
		}
		else
		{
			ccc_hValve2StatusFault_pre = ccc_hValve2StatusFault;
			EventNotifyCNT++;
			indicate_handle_no_reg(VALVE2_HANDLE_NUM_STATUSFAULT);
		}
	}
	session_timeout_tick = clock_time () | 1;
	return 1;
}

/* %% End Of File %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% */
