/********************************************************************************************************
 * @file     main.c
 *
 * @brief    for TLSR chips
 *
 * @author	 public@telink-semi.com;
 * @date     Sep. 18, 2018
 *
 * @par      Copyright (c) Telink Semiconductor (Shanghai) Co., Ltd.
 *           All rights reserved.
 *
 *			 The information contained herein is confidential and proprietary property of Telink
 * 		     Semiconductor (Shanghai) Co., Ltd. and is available under the terms
 *			 of Commercial License Agreement between Telink Semiconductor (Shanghai)
 *			 Co., Ltd. and the licensee in separate contract or the terms described here-in.
 *           This heading MUST NOT be removed from this file.
 *
 * 			 Licensees are granted free, non-transferable use of the information in this
 *			 file under Mutual Non-Disclosure Agreement. NO WARRENTY of ANY KIND is provided.
 *
 *******************************************************************************************************/

#include "tl_common.h"
#include "drivers.h"
#include "stack/ble/ble.h"

#include "../../vendor/common/user_config.h"

#include "./mfi/mfi.h"

#include "homekit_src/homekit_inc.h"
#include "./led_event/led_ctrl.h"
#include "tc_simulate/hk_tc_simulate.h"



extern void user_init_normal();
extern void user_init_deepRetn();
extern void cpu_wakeup_erase_ota_data(u32 firmware_size);

extern void main_loop (void);

extern void user_init_battery_power_check(void);

_attribute_ram_code_ void irq_handler(void)
{

	//hk_led_ctrl_proc();

	irq_blt_sdk_handler ();

	hk_led_ctrl_proc();

}





_attribute_ram_code_ int main (void)    //must run in ramcode
{

	DBG_CHN0_LOW;   //debug
	bls_ota_set_fwSize_and_fwBootAddr(OTA_PROGRAM_SIZE/1024,0x40000);

	blc_pm_select_internal_32k_crystal();

	cpu_wakeup_init();

	int deepRetWakeUp = pm_is_MCU_deepRetentionWakeup();  //MCU deep retention wakeUp

	rf_drv_init(RF_MODE_BLE_1M);

	gpio_init( !deepRetWakeUp );  //analog resistance will keep available in deepSleep mode, so no need initialize again

#if (CLOCK_SYS_CLOCK_HZ == 16000000)
	clock_init(SYS_CLK_16M_Crystal);
#elif (CLOCK_SYS_CLOCK_HZ == 24000000)
	clock_init(SYS_CLK_24M_Crystal);
#elif (CLOCK_SYS_CLOCK_HZ == 48000000)
	clock_init(SYS_CLK_48M_Crystal);
#endif

	if(!deepRetWakeUp){//read flash size
		unsigned int flash_mid=0;
		flash_mid_read01 ((u8 *)&flash_mid);
		if((flash_mid == 0x13325E)||(flash_mid == 0x14325E))
		{
			zbit_flash_flag = 1;
		}
		else
		{
			zbit_flash_flag = 0;
		}
	}
	if(zbit_flash_flag)
	{
		analog_write(0xc,(analog_read(0xc)&0xf8)|0x7);
	}

	blc_app_loadCustomizedParameters();  //load customized freq_offset cap value

	if( deepRetWakeUp ){
		user_init_deepRetn ();
	}
	else{
		user_init_battery_power_check(); //battery check must do before flash code
		cpu_wakeup_erase_ota_data(OTA_PROGRAM_SIZE);
		user_init_normal ();
	}

	HK_UART_OUT("homekit init finished.\r\n");
    irq_enable();

#if 0
/* check mfi id */
	u8 dev_id[4] = {0};
    if( app_verify_mfi_i2c(dev_id) )
    {
    	HK_DBG_OUT("MFI ID OK.\r\n");
    }
    else{
    	HK_DBG_OUT("MIF ID Fail.\r\n");
    }
#endif

	while (1) {
#if (MODULE_WATCHDOG_ENABLE)
		wd_clear(); //clear watch dog
#endif

#if HK_TEST_CASE_SIMULATE_ENABLE
		main_loop_simulate ();
#else
		main_loop ();
#endif
	}
}

